#include <arpa/inet.h>

#include <glib.h>

#include "lql-interface.h"
#include "lql-con.h"
#include "lql-qdisc-htb.h"
#include "lql-class-htb.h"
#include "lql-classifier-fwmark.h"

int
main(int argc,
     char *argv[])
{
	LQLCon *con = NULL;
	LQLQDiscHTB *qdisc = NULL;
	LQLClassHTB *htb = NULL;
	LQLInterface *interface = NULL;
	GList *list;
	LQLClassifierFwmark *fwmark1 = NULL;
	LQLClassifierFwmark *fwmark2 = NULL;

	g_type_init();

	con = (LQLCon *) lql_con_new();

	// Get the interfaces and find the index of eth0.
	list = lql_con_get_interfaces(con);
	while (list != NULL) {
		interface = list->data;
		if (g_strcasecmp("eth0", lql_interface_get_name(interface)) == 0) {
			break;
		} else {
			g_object_unref(interface);
		}

		list = list->next;
	}
	g_list_free(list);

	// Add the HTB QDisc.
	qdisc = (LQLQDiscHTB *)lql_qdisc_htb_new();
	lql_qdisc_set_parent(LQL_QDISC(qdisc), NULL);
	lql_qdisc_set_id(LQL_QDISC(qdisc), 1);
	lql_qdisc_htb_set_defaultClass(LQL_QDISC_HTB(qdisc), 1);
	lql_element_set_ifIndex(LQL_ELEMENT(qdisc), lql_interface_get_ifIndex(interface));
	if (!lql_qdisc_add(LQL_QDISC(qdisc), con)) {
		g_printerr("Could not add HTB QDisc.\n");
		return(-1);
	}

	// Add the HTB Class.
	htb = (LQLClassHTB *) lql_class_htb_new();
	lql_class_set_parent(LQL_CLASS(htb), LQL_ELEMENT(qdisc));
	lql_class_set_priority(LQL_CLASS(htb), 0);
	lql_class_set_id(LQL_CLASS(htb), 1);
	lql_element_set_ifIndex(LQL_ELEMENT(htb), lql_interface_get_ifIndex(interface));
	lql_class_htb_set_rate(htb, 10000000);
	lql_class_htb_set_ceiling(htb, 10000000);
	if (!lql_class_add(LQL_CLASS(htb), con)) {
		g_printerr("Could not add class.\n");
		return(-1);
	}

	/**
	 * This classifier is the root.
	 */
	fwmark1 = (LQLClassifierFwmark *)lql_classifier_fwmark_new();
	lql_classifier_set_priority(LQL_CLASSIFIER(fwmark1), 1);
	lql_classifier_set_interface(LQL_CLASSIFIER(fwmark1), LQL_INTERFACE(interface));
	if (!lql_classifier_add(LQL_CLASSIFIER(fwmark1), con)) {
		g_printerr("Could not add classifier.\n");
		return(-1);
	} else {
		g_print("Added fwmark 1.\n");
	}

	fwmark2 = (LQLClassifierFwmark *)lql_classifier_fwmark_new();
	lql_classifier_set_priority(LQL_CLASSIFIER(fwmark2), 5);
	lql_classifier_set_interface(LQL_CLASSIFIER(fwmark2), LQL_INTERFACE(interface));
	lql_classifier_set_class(LQL_CLASSIFIER(fwmark2), LQL_CLASS(htb));
	lql_classifier_set_parent(LQL_CLASSIFIER(fwmark2), LQL_ELEMENT(qdisc));
	lql_classifier_fwmark_set_mark(LQL_CLASSIFIER_FWMARK(fwmark2), 5);
	if (!lql_classifier_add(LQL_CLASSIFIER(fwmark2), con)) {
		g_printerr("Could not add classifier.\n");
		return(-1);
	} else {
		g_print("Added fwmark 2.\n");
	}

	g_object_unref(fwmark1);
	g_object_unref(fwmark2);
	g_object_unref(con);

	return(0);
}
