/*
 * This file implements part of the very massive filtering example in LARTC
 * http://www.lartc.org
 */
#include <arpa/inet.h>

#include <glib.h>

#include "lql-interface.h"
#include "lql-con.h"
#include "lql-qdisc.h"
#include "lql-qdisc-htb.h"
#include "lql-class.h"
#include "lql-class-htb.h"
#include "lql-classifier-u32.h"

int
main(int argc,
     char *argv[])
{
	LQLCon *con = NULL;
	LQLQDiscHTB *qdisc = NULL;
	LQLClassHTB *htb = NULL;
	LQLClassHTB *htb2 = NULL;
	LQLInterface *interface = NULL;
	LQLClassifierU32 *filt1 = NULL;
	LQLClassifierU32 *filt2 = NULL;
	LQLClassifierU32 *filt3 = NULL;
	LQLClassifierU32 *filt4 = NULL;
	GList *list;

	g_type_init();

	con = (LQLCon *) lql_con_new();

	// Get the interfaces and find the index of eth0.
	list = lql_con_get_interfaces(con);
	while (list != NULL) {
		interface = list->data;
		if (g_strcasecmp("eth0", lql_interface_get_name(interface)) == 0) {
			break;
		} else {
			g_object_unref(interface);
		}

		list = list->next;
	}
	g_list_free(list);

	// Add the HTB QDisc.
	qdisc = (LQLQDiscHTB *)lql_qdisc_htb_new();
	lql_qdisc_set_parent(LQL_QDISC(qdisc), NULL);
	lql_qdisc_set_id(LQL_QDISC(qdisc), 1);
	lql_qdisc_htb_set_defaultClass(LQL_QDISC_HTB(qdisc), 1);
	lql_element_set_ifIndex(LQL_ELEMENT(qdisc), lql_interface_get_ifIndex(interface));
	if (!lql_qdisc_add(LQL_QDISC(qdisc), con)) {
		g_printerr("Could not add HTB QDisc.\n");
		return(-1);
	}

	// Add the HTB Class.
	htb = (LQLClassHTB *) lql_class_htb_new();
	lql_class_set_parent(LQL_CLASS(htb), LQL_ELEMENT(qdisc));
	lql_class_set_priority(LQL_CLASS(htb), 0);
	lql_class_set_id(LQL_CLASS(htb), 1);
	lql_element_set_ifIndex(LQL_ELEMENT(htb), lql_interface_get_ifIndex(interface));
	lql_class_htb_set_rate(htb, 10240);
	lql_class_htb_set_ceiling(htb, 10240);
	if (!lql_class_add(LQL_CLASS(htb), con)) {
		g_printerr("Could not add class.\n");
		return(-1);
	}

	// Add the second HTB Class.
	htb2 = (LQLClassHTB *) lql_class_htb_new();
	lql_class_set_parent(LQL_CLASS(htb2), LQL_ELEMENT(qdisc));
	lql_class_set_priority(LQL_CLASS(htb2), 0);
	lql_class_set_id(LQL_CLASS(htb2), 2);
	lql_element_set_ifIndex(LQL_ELEMENT(htb2), lql_interface_get_ifIndex(interface));
	lql_class_htb_set_rate(htb2, 1000);
	lql_class_htb_set_ceiling(htb2, 1000);
	if (!lql_class_add(LQL_CLASS(htb2), con)) {
		g_printerr("Could not add class.\n");
		return(-1);
	}

	// Add the base filter.
	g_print("--- Adding the base filter ---\n");
	filt1 = (LQLClassifierU32 *)lql_classifier_u32_new();
	lql_classifier_set_priority(LQL_CLASSIFIER(filt1), 5);
	lql_classifier_set_ifIndex(LQL_CLASSIFIER(filt1),
				   lql_interface_get_ifIndex(interface));
	lql_classifier_set_parent(LQL_CLASSIFIER(filt1), LQL_ELEMENT(qdisc));
	if (!lql_classifier_add(LQL_CLASSIFIER(filt1), con)) {
		g_printerr("Could not add first classifier.\n");
		return(-1);
	}

	g_print("--- Adding the second filter ---\n");
	filt2 = (LQLClassifierU32 *)lql_classifier_u32_new();
	lql_classifier_set_priority(LQL_CLASSIFIER(filt2), 5);
	lql_classifier_set_ifIndex(LQL_CLASSIFIER(filt2),
				   lql_interface_get_ifIndex(interface));
	lql_classifier_set_parent(LQL_CLASSIFIER(filt2), LQL_ELEMENT(qdisc));
	lql_classifier_u32_set_handle(LQL_CLASSIFIER_U32(filt2), 3);
	lql_classifier_u32_set_divisor(LQL_CLASSIFIER_U32(filt2), 256);
	if (!lql_classifier_add(LQL_CLASSIFIER(filt2), con)) {
		g_printerr("Could not add second classifier.\n");
		return(-1);
	}

	g_print("--- Adding the third filter ---\n");
	filt3 = (LQLClassifierU32 *)lql_classifier_u32_new();
	lql_classifier_set_priority(LQL_CLASSIFIER(filt3), 5);
	lql_classifier_set_ifIndex(LQL_CLASSIFIER(filt3),
				   lql_interface_get_ifIndex(interface));
	lql_classifier_set_class(LQL_CLASSIFIER(filt3), LQL_CLASS(htb2));
	lql_classifier_set_parent(LQL_CLASSIFIER(filt3), LQL_ELEMENT(qdisc));
	lql_classifier_u32_set_handle(LQL_CLASSIFIER_U32(filt3), 3);
	lql_classifier_u32_set_hashHandle(LQL_CLASSIFIER_U32(filt3), 9);
	lql_classifier_u32_set_hashNode(LQL_CLASSIFIER_U32(filt3), 1);
	if (!lql_classifier_u32_add_match_ip_dst(filt3, "69.41.199.9")) {
		g_printerr("Could not add match to third classifier.\n");
		return(-1);
	}
	if (!lql_classifier_add(LQL_CLASSIFIER(filt3), con)) {
		g_printerr("Could not add third classifier.\n");
		return(-1);
	}

	g_print("--- Adding the fourth filter ---\n");
	filt4 = (LQLClassifierU32 *)lql_classifier_u32_new();
	lql_classifier_set_priority(LQL_CLASSIFIER(filt4), 5);
	lql_classifier_set_ifIndex(LQL_CLASSIFIER(filt4),
				   lql_interface_get_ifIndex(interface));
	lql_classifier_set_parent(LQL_CLASSIFIER(filt4), LQL_ELEMENT(qdisc));
	lql_classifier_u32_set_handle(LQL_CLASSIFIER_U32(filt4), 0x800);
	lql_classifier_u32_set_hashNode(LQL_CLASSIFIER_U32(filt4), 0x800);
	lql_classifier_u32_set_hashMask(LQL_CLASSIFIER_U32(filt4), 0x000000FF);
	lql_classifier_u32_set_hashOffset(LQL_CLASSIFIER_U32(filt4), 16);
	lql_classifier_u32_set_link(LQL_CLASSIFIER_U32(filt4),
				    lql_classifier_get_id(LQL_CLASSIFIER(filt2)));
	if (!lql_classifier_u32_add_match_ip_dst(filt4, "69.41.199.9")) {
		g_printerr("Could not add match to fourth classifier.\n");
		return(-1);
	}
	if (!lql_classifier_add(LQL_CLASSIFIER(filt4), con)) {
		g_printerr("Could not add fourth classifier.\n");
		return(-1);
	}

	g_object_unref(filt1);
	g_object_unref(filt2);
	g_object_unref(filt3);
	g_object_unref(htb2);
	g_object_unref(htb);
	g_object_unref(qdisc);
	g_object_unref(con);

	return(0);
}
