/*
 * example1.c
 *
 * This example adds a root HTB QDisc and 4 classes to this QDisc. It then fetches the list
 * of classes from the system and modifies the fourth class and deletes the third.
 */
#include <arpa/inet.h>
#include <unistd.h>

#include <glib.h>

#include "lql-interface.h"
#include "lql-con.h"
#include "lql-qdisc.h"
#include "lql-qdisc-htb.h"
#include "lql-class.h"
#include "lql-class-htb.h"
#include "lql-classifier-u32.h"

#define INTERFACE "eth0"
#define DELAY 5

int
main(int argc,
     char *argv[])
{
	LQLCon *con = NULL;
	LQLQDiscHTB *qdisc = NULL;
	LQLClass *class = NULL;
	LQLClassHTB *htb = NULL;
	LQLClassHTB *htb2 = NULL;
	LQLClassHTB *htb3 = NULL;
	LQLClassHTB *htb4 = NULL;
	LQLInterface *interface = NULL;
	GList *list;

	g_type_init();

	con = (LQLCon *) lql_con_new();

	// Get the interfaces and find the index of eth0.
	list = lql_con_get_interfaces(con);
	while (list != NULL) {
		interface = list->data;
		if (g_strcasecmp(INTERFACE, lql_interface_get_name(interface)) == 0) {
			break;
		} else {
			g_object_unref(interface);
		}

		list = list->next;
	}
	g_list_free(list);

	// Add the HTB QDisc.
	qdisc = (LQLQDiscHTB *)lql_qdisc_htb_new();
	lql_qdisc_set_parent(LQL_QDISC(qdisc), NULL);
	lql_qdisc_set_id(LQL_QDISC(qdisc), 1);
	lql_qdisc_htb_set_defaultClass(LQL_QDISC_HTB(qdisc), 1);
	lql_element_set_ifIndex(LQL_ELEMENT(qdisc), lql_interface_get_ifIndex(interface));
	if (!lql_qdisc_add(LQL_QDISC(qdisc), con)) {
		g_printerr("Could not add HTB QDisc.\n");
		return(-1);
	}

	// Add the HTB Class.
	htb = (LQLClassHTB *) lql_class_htb_new();
	lql_class_set_parent(LQL_CLASS(htb), LQL_ELEMENT(qdisc));
	lql_class_set_priority(LQL_CLASS(htb), 0);
	lql_class_set_id(LQL_CLASS(htb), 1);
	lql_element_set_ifIndex(LQL_ELEMENT(htb), lql_interface_get_ifIndex(interface));
	lql_class_htb_set_rate(htb, 10240);
	lql_class_htb_set_ceiling(htb, 10240);
	if (!lql_class_add(LQL_CLASS(htb), con)) {
		g_printerr("Could not add class.\n");
		return(-1);
	}

	// Add the second HTB Class.
	htb2 = (LQLClassHTB *) lql_class_htb_new();
	lql_class_set_parent(LQL_CLASS(htb2), LQL_ELEMENT(qdisc));
	g_print("Parent: %u\n", lql_qdisc_get_id(LQL_QDISC(qdisc)));
	lql_class_set_priority(LQL_CLASS(htb2), 0);
	lql_class_set_id(LQL_CLASS(htb2), 2);
	lql_element_set_ifIndex(LQL_ELEMENT(htb2), lql_interface_get_ifIndex(interface));
	lql_class_htb_set_rate(htb2, 8000);
	lql_class_htb_set_ceiling(htb2, 8000);
	if (!lql_class_add(LQL_CLASS(htb2), con)) {
		g_printerr("Could not add class.\n");
		return(-1);
	}

	// Add the third HTB Class.
	htb3 = (LQLClassHTB *) lql_class_htb_new();
	lql_class_set_parent(LQL_CLASS(htb3), LQL_ELEMENT(qdisc));
	lql_class_set_priority(LQL_CLASS(htb3), 0);
	lql_class_set_id(LQL_CLASS(htb3), 3);
	lql_element_set_ifIndex(LQL_ELEMENT(htb3), lql_interface_get_ifIndex(interface));
	lql_class_htb_set_rate(htb3, 4000);
	lql_class_htb_set_ceiling(htb3, 4000);
	if (!lql_class_add(LQL_CLASS(htb3), con)) {
		g_printerr("Could not add class.\n");
		return(-1);
	}

	// Add the fourth HTB Class.
	htb4 = (LQLClassHTB *) lql_class_htb_new();
	lql_class_set_parent(LQL_CLASS(htb4), LQL_ELEMENT(qdisc));
	lql_class_set_priority(LQL_CLASS(htb4), 0);
	lql_class_set_id(LQL_CLASS(htb4), 4);
	lql_element_set_ifIndex(LQL_ELEMENT(htb4), lql_interface_get_ifIndex(interface));
	lql_class_htb_set_rate(htb4, 2000);
	lql_class_htb_set_ceiling(htb4, 2000);
	if (!lql_class_add(LQL_CLASS(htb4), con)) {
		g_printerr("Could not add class.\n");
		return(-1);
	}

	g_object_unref(htb4);
	g_object_unref(htb3);
	g_object_unref(htb2);
	g_object_unref(htb);
	g_object_unref(qdisc);

	g_print("QDisc and classes have been added. Sleeping for %i seconds.\n", DELAY);
	sleep(DELAY);

	list = lql_con_get_classes(con, interface);
	while (list != NULL) {
		class = (LQLClass *)list->data;
		if (!LQL_IS_CLASS_HTB(class)) {
			g_print("This program only works with HTB classes.\n");
		}
		if (lql_class_get_id(LQL_CLASS(class)) == 3) {
			g_print("Deleting class 3.\n");
			if (!lql_class_delete(LQL_CLASS(class), LQL_CON(con)))
				g_print("Could not delete class 3.\n");
		} else if (lql_class_get_id(LQL_CLASS(class)) == 4) {
			lql_class_htb_set_rate(LQL_CLASS_HTB(class), 2222);
			if (!lql_class_modify(LQL_CLASS(class), LQL_CON(con))) {
				g_print("Modifying class 4.\n");
			}
		}

		g_object_unref(class);

		list = list->next;
	}
	g_list_free(list);

	g_object_unref(interface);
	g_object_unref(con);

	return(0);
}
