/*
 * This program will list the network QoS elements in the system and their parameters.
 */
#include <unistd.h>
#include <stdlib.h>

#include <glib.h>

#include "lql-con.h"
#include "lql-qdisc.h"
#include "lql-qdisc-htb.h"
#include "lql-qdisc-sfq.h"
#include "lql-qdisc-pfifofast.h"
#include "lql-qdisc-pfifo.h"
#include "lql-qdisc-priomap.h"
#include "lql-class.h"
#include "lql-class-htb.h"
#include "lql-class-priomap.h"
#include "lql-classifier.h"
#include "lql-classifier-u32.h"
#include "lql-classifier-fwmark.h"
#include "lql-interface.h"

void
print_classifier_u32(LQLClassifierU32 *classifier)
{
	g_print("U32: handle: %u\n",
		lql_classifier_u32_get_handle(LQL_CLASSIFIER_U32(classifier)));
	g_print("U32: hashHandle: %u\n",
		lql_classifier_u32_get_hashHandle(LQL_CLASSIFIER_U32(classifier)));
	g_print("U32: hashNode: %u\n",
		lql_classifier_u32_get_hashNode(LQL_CLASSIFIER_U32(classifier)));
	g_print("U32: divisor: %u\n",
		lql_classifier_u32_get_divisor(LQL_CLASSIFIER_U32(classifier)));
	g_print("U32: hash: %u\n",
		lql_classifier_u32_get_hash(LQL_CLASSIFIER_U32(classifier)));
	g_print("U32: link: %u\n",
		lql_classifier_u32_get_link(LQL_CLASSIFIER_U32(classifier)));
}

void
print_classifier_fwmark(LQLClassifierFwmark *classifier)
{
	g_print("Fwmark: mark: %u\n", lql_classifier_get_id(LQL_CLASSIFIER(classifier)));
}

int
main(int argc,
     char *argv[])
{
	LQLCon *con = NULL;
	LQLInterface *interface = NULL;
	GList *list;

	g_type_init();

	con = (LQLCon *)lql_con_new();

	// Get the interfaces.
	interface = lql_con_get_interface_by_name(con, "eth0");

	// Get the number of QDiscs.
	g_print("-------------------------------------\n");
	list = lql_con_get_qdiscs(con, interface);
	g_print("Number of QDiscs: %i\n", g_list_length(list));
	while (list != NULL) {
		LQLQDisc *qdisc = NULL;
		qdisc = (LQLQDisc *)list->data;

		if (qdisc != NULL) {
			lql_qdisc_print(qdisc);
			
			g_object_unref(qdisc);
		} else {
			g_print("NULL QDisc\n");
		}

		list = list->next;
	}
	g_list_free(list);

	// Get the number of Classes.
	g_print("-------------------------------------\n");
	list = lql_con_get_classes(con, interface);
	g_print("Number of classes: %i\n", g_list_length(list));
	while (list != NULL) {
		LQLClass *class = NULL;
		class = (LQLClass *)list->data;

		if (class != NULL) {
			lql_class_print(class);

			g_object_unref(class);
		} else {
			g_print("NULL class.\n");
		}
		
		list = list->next;
	}
	g_list_free(list);

	// Get the classifiers
	g_print("-------------------------------------\n");
	list = lql_con_get_classifiers(con, interface);
	g_print("Number of classifiers: %i\n", g_list_length(list));
	while (list != NULL) {
		LQLClassifier *classifier = NULL;
		classifier = (LQLClassifier *)list->data;
		g_print("---------CLASSIFIER------------\n");
		if (classifier != NULL) {
			g_print("ID: %u\n", lql_classifier_get_id(classifier));
			g_print("Parent: %i\n", lql_classifier_get_parentHandle(classifier));
			g_print("ifIndex: %i\n", lql_classifier_get_ifIndex(classifier));
			g_print("Class: %i\n", lql_classifier_get_classHandle(classifier));
			g_print("Priority: %i\n", lql_classifier_get_priority(classifier));
			g_print("Protocol: %i\n", lql_classifier_get_protocol(classifier));
			if (LQL_IS_CLASSIFIER_FWMARK(classifier)) {
				print_classifier_fwmark(LQL_CLASSIFIER_FWMARK(classifier));
			} else if (LQL_IS_CLASSIFIER_U32(classifier)) {
				print_classifier_u32(LQL_CLASSIFIER_U32(classifier));
			} else {
				g_print("get_test UNKNOWN CLASSIFIER\n");
			}

			g_object_unref(classifier);
		} else {
			g_print("NULL classifier.\n");
		}

		list = list->next;
	}
	g_list_free(list);

	// Cleanup.
	g_object_unref(interface);
	g_object_unref(con);

	return(0);
}
