/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include "lql-class-dsmark.h"
#include "lql-class-dsmark-private.h"
#include "lql_class_dsmark_helper.h"

/********************************************************************
 * Non-Virutal API
 ********************************************************************/

/**
 * lql_class_dsmark_new:
 *
 * Build a new instance of LQLClassDSMark. 
 * 
 * Return value: A new LQLClassDSMark instance.
 **/
LQLClassDSMark*
lql_class_dsmark_new(void)
{
        return(LQL_CLASS_DSMARK(g_object_new(LQL_TYPE_CLASS_DSMARK, NULL)));
}

/**
 * lql_class_dsmark_get_mask:
 * @self: The LQLClassDSMark instance.
 * 
 * Get the mask value for this class.
 * 
 * Return value: The mask value.
 **/
guint8
lql_class_dsmark_get_mask(LQLClassDSMark *self)
{
	guint32 mask = 0;
	
	g_assert(self != NULL);
	
        if (self->private->dispose_has_run) {
                return(-1);
        }

	g_object_get(LQL_CLASS_DSMARK(self), "mask", &mask, NULL);

        return(mask);
}

/**
 * lql_class_dsmark_set_mask:
 * @self: The LQLClassDSMark instance.
 * @mask: The new mask value.
 * 
 * Set the mask value for this class.
 * 
 * Return value: TRUE on success, FALSE on error.
 **/
gboolean
lql_class_dsmark_set_mask(LQLClassDSMark *self,
			  guint8 mask)
{
	g_assert(self != NULL);

	if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASS_DSMARK(self), "mask", mask, NULL);
        
        return(TRUE);
}

/**
 * lql_class_dsmark_get_value:
 * @self: The LQLClassDSMark instance.
 * 
 * Get the value parameter for this class.
 * 
 * Return value: The value parameter.
 **/
guint8
lql_class_dsmark_get_value(LQLClassDSMark *self)
{
	guint32 value = 0;
	
	g_assert(self != NULL);
	
        if (self->private->dispose_has_run) {
                return(-1);
        }

	g_object_get(LQL_CLASS_DSMARK(self), "value", &value, NULL);

        return(value);
}


/**
 * lql_class_dsmark_set_value:
 * @self: The LQLClassDSMark instance.
 * @value: The new value.
 * 
 * Set the value parameter for this class.
 * 
 * Return value: TRUE on success, FALSE on failure.
 **/
gboolean
lql_class_dsmark_set_value(LQLClassDSMark *self,
			   guint8 value)
{
	g_assert(self != NULL);

	if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASS_DSMARK(self), "value", value, NULL);
        
        return(TRUE);
}

/********************************************************************
 * Virtual API
 ********************************************************************/

/**
 * lql_class_dsmark_fill_from_nlhdr:
 * @self: The LQLClass object to work with.
 * @hdr: The nl packet to get the queue information from.
 *
 * This method takes a nlmsg from the kernel and set this
 * objects instance variables to match it. This is used
 * to create objects that match the ones currently active
 * in the system. This overrides the default LQLClass
 * implementation of this method.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_class_dsmark_fill_from_nlhdr(LQLClass *self,
				 const struct nlmsghdr *hdr)
{
	g_assert(self != NULL);
	g_assert(hdr != NULL);

	return(lql_class_dsmark_helper_fill_from_nlhdr(LQL_CLASS_DSMARK(self), hdr));
}

/**
 * lql_class_dsmark_add:
 * @self: The LQLClass object to add to the system.
 * @con: The LQLCon connection to use when adding this queueing discipline.
 *
 * Add this DSMark queueing discipline to the system. This overrides
 * the default implementation of this function in LQLClass.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_class_dsmark_add(LQLClass *self,
		     LQLCon *con)
{
	g_assert(self != NULL);
	g_assert(con != NULL);

	return(lql_class_dsmark_helper_add(LQL_CLASS_DSMARK(self), con));
}

/**
 * lql_class_dsmark_modify:
 * @self: The LQLClass object to modify.
 * @con: The LQLCon connection to use when adding this class.
 *
 * Modify the DSMark queueing discipline in the system to match the current
 * object state. This method overrides the default
 * implementation in LQLClass.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_class_dsmark_modify(LQLClass *self,
			LQLCon *con)
{
	g_assert(self != NULL);
	g_assert(con != NULL);

	return(lql_class_dsmark_helper_modify(LQL_CLASS_DSMARK(self), con));
}

/********************************************************************
 * Object stuff.
 ********************************************************************/
enum {
        LQL_CLASS_DSMARK_MASK = 1,
        LQL_CLASS_DSMARK_VALUE = 2,
};

static void
lql_class_dsmark_instance_init(GTypeInstance *instance,
			       gpointer g_class)
{
	LQLClassDSMark *self = (LQLClassDSMark *)instance;
	self->private = g_new0(LQLClassDSMarkPrivate, 1);
	self->private->dispose_has_run = FALSE;

	self->private->mask = 0;
	self->private->value = 0;
}

static void
lql_class_dsmark_set_property(GObject *object,
			      guint property_id,
			      const GValue *value,
			      GParamSpec *pspec)
{
        LQLClassDSMark *self = (LQLClassDSMark *) object;

        switch (property_id) {
        case LQL_CLASS_DSMARK_MASK:
        {
                self->private->mask = g_value_get_uint(value);
                break;
        }
        case LQL_CLASS_DSMARK_VALUE:
        {
                self->private->value = g_value_get_uint(value);
                break;
        }
        default:
                g_assert(FALSE);
                break;
        }
}

static void
lql_class_dsmark_get_property(GObject *object,
			      guint property_id,
			      GValue *value,
			      GParamSpec *pspec)
{
        LQLClassDSMark *self = (LQLClassDSMark *) object;

        switch (property_id) {
        case LQL_CLASS_DSMARK_MASK:
        {
                g_value_set_uint(value, self->private->mask);
                break;
        }
        case LQL_CLASS_DSMARK_VALUE:
        {
                g_value_set_uint(value, self->private->value);
                break;
        }
        default:
                g_assert(FALSE);
                break;
        }
}

static GObject *
lql_class_dsmark_constructor(GType type,
			     guint n_construct_properties,
			     GObjectConstructParam *construct_properties)
{
        GObject *obj = NULL;

        {
                /* Invoke parent constructor. */
		LQLClassDSMarkClass *klass;
		GObjectClass *parent_class;  
		klass = LQL_CLASS_DSMARK_CLASS(g_type_class_peek(LQL_TYPE_CLASS_DSMARK));
		parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));
		obj = parent_class->constructor(type,
						n_construct_properties,
						construct_properties);
        }
        
        /* do stuff. */

        return obj;
}

static void
lql_class_dsmark_dispose(GObject *obj)
{
        LQLClassDSMark *self = (LQLClassDSMark *)obj;

        if (self->private->dispose_has_run) {
                /* If dispose did already run, return. */
                return;
        }
        /* Make sure dispose does not run twice. */
        self->private->dispose_has_run = TRUE;

        /* 
         * In dispose, you are supposed to free all types referenced from this
         * object which might themselves hold a reference to self. Generally,
         * the most simple solution is to unref all members on which you own a 
         * reference.
         */
}

static void
lql_class_dsmark_finalize(GObject *obj)
{
        LQLClassDSMark *self = (LQLClassDSMark *)obj;

        /*
         * Here, complete object destruction.
         */
        
        g_free (self->private);
}


static void
lql_class_dsmark_class_init(gpointer g_class,
			    gpointer g_class_data)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS(g_class);
        LQLClassClass *klass = LQL_CLASS_CLASS(g_class);
        GParamSpec *pspec = NULL;

        gobject_class->set_property = lql_class_dsmark_set_property;
        gobject_class->get_property = lql_class_dsmark_get_property;
        gobject_class->dispose = lql_class_dsmark_dispose;
        gobject_class->finalize = lql_class_dsmark_finalize;
        gobject_class->constructor = lql_class_dsmark_constructor;

	/* Set the virtual methods. */
	klass->fill_from_nlhdr = lql_class_dsmark_fill_from_nlhdr;
	klass->add = lql_class_dsmark_add;
	klass->modify = lql_class_dsmark_modify;
	klass->print = lql_class_dsmark_helper_print;
	klass->printStats = lql_class_dsmark_helper_printStats;
	klass->updateStats = lql_class_dsmark_helper_updateStats;

	/* Setup the properties. */
        pspec = g_param_spec_uint("mask",
				  "The mask value.",
				  "Set/Get the mask value for this class.",
				  0 /* minimum value */,
				  G_MAXUINT8 /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASS_DSMARK_MASK,
                                        pspec);

        pspec = g_param_spec_uint("value",
				  "The value for this class.",
				  "Set/Get the value for this class.",
				  0 /* minimum value */,
				  G_MAXUINT8 /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASS_DSMARK_VALUE,
                                        pspec);
}

GType lql_class_dsmark_get_type (void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (LQLClassDSMarkClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        lql_class_dsmark_class_init,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (LQLClassDSMark),
                        0,      /* n_preallocs */
                        lql_class_dsmark_instance_init    /* instance_init */
                };
                type = g_type_register_static (LQL_TYPE_CLASS,
                                               "LQLClassDSMark",
                                               &info, 0);
        }
        return type;
}
