/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include "lql-class-htb.h"
#include "lql-class-htb-private.h"
#include "lql_class_htb_helper.h"

/********************************************************************
 * Non-Virutal API
 ********************************************************************/

/**
 * lql_class_htb_new:
 *
 * Build a new instance of LQLClassHTB. 
 * 
 * Return value: A new LQLClassHTB instance.
 **/
LQLClassHTB*
lql_class_htb_new(void)
{
        return(LQL_CLASS_HTB(g_object_new(LQL_TYPE_CLASS_HTB, NULL)));
}

/**
 * lql_class_htb_get_rate:
 * @self: The LQLClassHTB instance.
 * 
 * Get the rate of this class in bytes per second (bps).
 * 
 * Return value: The rate in bps.
 **/
unsigned int
lql_class_htb_get_rate(LQLClassHTB *self)
{
	unsigned int rate = 0;
	
	g_assert(self != NULL);
	
        if (self->private->dispose_has_run) {
                return(-1);
        }

	g_object_get(LQL_CLASS_HTB(self), "rate", &rate, NULL);

        return(rate);
}

/**
 * lql_class_htb_set_rate:
 * @self: The LQLClassHTB instance.
 * @rate: The rate in bytes per second to set.
 * 
 * Set the rate of this class in bytes per second (bps).
 * 
 * Return value: TRUE on success, FALSE on failure.
 **/
gboolean
lql_class_htb_set_rate(LQLClassHTB *self,
		       unsigned int rate)
{
	g_assert(self != NULL);

	if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASS_HTB(self), "rate", rate, NULL);
        
        return(TRUE);
}

/**
 * lql_class_htb_get_ceiling:
 * @self: The LQLClassHTB instance.
 * 
 * Get the ceiling rate for this class in bytes per second (bps).
 * 
 * Return value: The ceiling rate for this class in bps.
 **/
unsigned int
lql_class_htb_get_ceiling(LQLClassHTB *self)
{
	unsigned int ceiling = 0;
	
	g_assert(self != NULL);
	
        if (self->private->dispose_has_run) {
                return(-1);
        }

	g_object_get(LQL_CLASS_HTB(self), "ceiling", &ceiling, NULL);

        return(ceiling);
}

/**
 * lql_class_htb_set_ceiling:
 * @self: The LQLClassHTB instance.
 * @ceiling: The ceiling rate for this class.
 * 
 * Set the ceiling rate for this class in bytes per second (bps).
 * 
 * Return value: The ceiling rate for this class in bps.
 **/
gboolean
lql_class_htb_set_ceiling(LQLClassHTB *self,
			  unsigned int ceiling)
{
	g_assert(self != NULL);

	if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASS_HTB(self), "ceiling", ceiling, NULL);
        
        return(TRUE);
}

/**
 * lql_class_htb_get_mtu:
 * @self: The LQLClassHTB instance.
 * 
 * Get the MTU value in bytes that is being used by this LQLClassHTB.
 * 
 * Return value: The MTU value in bytes.
 **/
unsigned int
lql_class_htb_get_mtu(LQLClassHTB *self)
{
	unsigned int mtu = 0;
	
	g_assert(self != NULL);
	
        if (self->private->dispose_has_run) {
                return(-1);
        }

	g_object_get(LQL_CLASS_HTB(self), "mtu", &mtu, NULL);

        return(mtu);
}

/**
 * lql_class_htb_set_mtu:
 * @self: The LQLClassHTB instance.
 * @mtu: The MTU in bytes.
 * 
 * Set the MTU value in bytes that is being used by this LQLClassHTB.
 * 
 * Return value: The MTU value.
 **/
gboolean
lql_class_htb_set_mtu(LQLClassHTB *self,
		      unsigned int mtu)
{
	g_assert(self != NULL);

	if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASS_HTB(self), "mtu", mtu, NULL);
        
        return(TRUE);
}

/**
 * lql_class_htb_get_mpu:
 * @self: The LQLClassHTB instance.
 * 
 * Get the MPU value in bytes being used by this LQLClassHTB.
 * 
 * Return value: The MPU value in bytes.
 **/
unsigned int
lql_class_htb_get_mpu(LQLClassHTB *self)
{
	unsigned int mpu = 0;
	
	g_assert(self != NULL);
	
        if (self->private->dispose_has_run) {
                return(-1);
        }

	g_object_get(LQL_CLASS_HTB(self), "mpu", &mpu, NULL);

        return(mpu);
}

/**
 * lql_class_htb_set_mpu:
 * @self: The LQLClassHTB instance.
 * @mpu: The MPU value in bytes.
 * 
 * Set the MPU value in bytes that is being used by this LQLClassHTB.
 * 
 * Return value: TRUE on success, FALSE on error.
 **/
gboolean
lql_class_htb_set_mpu(LQLClassHTB *self,
		      unsigned int mpu)
{
	g_assert(self != NULL);

	if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASS_HTB(self), "mpu", mpu, NULL);
        
        return(TRUE);
}

/********************************************************************
 * Virtual API
 ********************************************************************/

/**
 * lql_class_htb_fill_from_nlhdr:
 * @self: The LQLClass object to work with.
 * @hdr: The nl packet to get the queue information from.
 *
 * This method takes a nlmsg from the kernel and set this
 * objects instance variables to match it. This is used
 * to create objects that match the ones currently active
 * in the system. This overrides the default LQLClass
 * implementation of this method.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_class_htb_fill_from_nlhdr(LQLClass *self,
			      const struct nlmsghdr *hdr)
{
	g_assert(self != NULL);
	g_assert(hdr != NULL);

	return(lql_class_htb_helper_fill_from_nlhdr(LQL_CLASS_HTB(self), hdr));
}

/**
 * lql_class_htb_add:
 * @self: The LQLClass object to add to the system.
 * @con: The LQLCon connection to use when adding this queueing discipline.
 *
 * Add this HTB queueing discipline to the system. This overrides
 * the default implementation of this function in LQLClass.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_class_htb_add(LQLClass *self,
		  LQLCon *con)
{
	g_assert(self != NULL);
	g_assert(con != NULL);

	return(lql_class_htb_helper_add(LQL_CLASS_HTB(self), con));
}

/**
 * lql_class_htb_modify:
 * @self: The LQLClass object to modify.
 * @con: The LQLCon connection to use when adding this class.
 *
 * Modify the HTB queueing discipline in the system to match the current
 * object state. This method overrides the default
 * implementation in LQLClass.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_class_htb_modify(LQLClass *self,
		     LQLCon *con)
{
	g_assert(self != NULL);
	g_assert(con != NULL);

	return(lql_class_htb_helper_modify(LQL_CLASS_HTB(self), con));
}

/********************************************************************
 * Object stuff.
 ********************************************************************/
enum {
        LQL_CLASS_HTB_RATE = 1,
        LQL_CLASS_HTB_CEILING = 2,
	LQL_CLASS_HTB_MTU = 3,
	LQL_CLASS_HTB_MPU = 4,
};

static void
lql_class_htb_instance_init(GTypeInstance *instance,
			    gpointer g_class)
{
	LQLClassHTB *self = (LQLClassHTB *)instance;
	self->private = g_new0(LQLClassHTBPrivate, 1);
	self->private->dispose_has_run = FALSE;

	self->private->mtu = 1600;
	self->private->mpu = 50;
}

static void
lql_class_htb_set_property(GObject *object,
			   guint property_id,
			   const GValue *value,
			   GParamSpec *pspec)
{
        LQLClassHTB *self = (LQLClassHTB *) object;

        switch (property_id) {
        case LQL_CLASS_HTB_RATE:
        {
                self->private->rate = g_value_get_uint(value);
                break;
        }
        case LQL_CLASS_HTB_CEILING:
        {
                self->private->ceiling = g_value_get_uint(value);
                break;
        }
        case LQL_CLASS_HTB_MTU:
        {
                self->private->mtu = g_value_get_uint(value);
                break;
        }
        case LQL_CLASS_HTB_MPU:
        {
                self->private->mpu = g_value_get_uint(value);
                break;
        }

        default:
                g_assert(FALSE);
                break;
        }
}

static void
lql_class_htb_get_property(GObject *object,
                           guint property_id,
                           GValue *value,
                           GParamSpec *pspec)
{
        LQLClassHTB *self = (LQLClassHTB *) object;

        switch (property_id) {
        case LQL_CLASS_HTB_RATE:
        {
                g_value_set_uint(value, self->private->rate);
                break;
        }
        case LQL_CLASS_HTB_CEILING:
        {
                g_value_set_uint(value, self->private->ceiling);
                break;
        }
        case LQL_CLASS_HTB_MTU:
        {
                g_value_set_uint(value, self->private->mtu);
                break;
        }
        case LQL_CLASS_HTB_MPU:
        {
                g_value_set_uint(value, self->private->mpu);
                break;
        }

        default:
                g_assert(FALSE);
                break;
        }
}

static GObject *
lql_class_htb_constructor(GType type,
                          guint n_construct_properties,
                          GObjectConstructParam *construct_properties)
{
        GObject *obj = NULL;

        {
                /* Invoke parent constructor. */
		LQLClassHTBClass *klass;
		GObjectClass *parent_class;  
		klass = LQL_CLASS_HTB_CLASS(g_type_class_peek(LQL_TYPE_CLASS_HTB));
		parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));
		obj = parent_class->constructor(type,
						n_construct_properties,
						construct_properties);
        }
        
        /* do stuff. */

        return obj;
}

static void
lql_class_htb_dispose(GObject *obj)
{
        LQLClassHTB *self = (LQLClassHTB *)obj;

        if (self->private->dispose_has_run) {
                /* If dispose did already run, return. */
                return;
        }
        /* Make sure dispose does not run twice. */
        self->private->dispose_has_run = TRUE;

        /* 
         * In dispose, you are supposed to free all types referenced from this
         * object which might themselves hold a reference to self. Generally,
         * the most simple solution is to unref all members on which you own a 
         * reference.
         */
}

static void
lql_class_htb_finalize(GObject *obj)
{
        LQLClassHTB *self = (LQLClassHTB *)obj;

        /*
         * Here, complete object destruction.
         */

        
        g_free (self->private);
}


static void
lql_class_htb_class_init(gpointer g_class,
			 gpointer g_class_data)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS(g_class);
        LQLClassClass *klass = LQL_CLASS_CLASS(g_class);
        GParamSpec *pspec = NULL;

        gobject_class->set_property = lql_class_htb_set_property;
        gobject_class->get_property = lql_class_htb_get_property;
        gobject_class->dispose = lql_class_htb_dispose;
        gobject_class->finalize = lql_class_htb_finalize;
        gobject_class->constructor = lql_class_htb_constructor;

	/* Set the virtual methods. */
	klass->fill_from_nlhdr = lql_class_htb_fill_from_nlhdr;
	klass->add = lql_class_htb_add;
	klass->modify = lql_class_htb_modify;
	klass->print = lql_class_htb_helper_print;
	klass->printStats = lql_class_htb_helper_printStats;
	klass->updateStats = lql_class_htb_helper_updateStats;

	/* Setup the properties. */
        pspec = g_param_spec_uint("rate",
				  "The rate in bytes/sec for this class",
				  "Set/Get the rate in bytes/sec for this class",
				  0 /* minimum value */,
				  G_MAXUINT /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASS_HTB_RATE,
                                        pspec);

        pspec = g_param_spec_uint("ceiling",
				  "The ceiling rate in bytes/sec for this class",
				  "Set/Get the ceiling rate in bytes/sec for this class",
				  0 /* minimum value */,
				  G_MAXUINT /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASS_HTB_CEILING,
                                        pspec);

        pspec = g_param_spec_uint("mtu",
				  "The MTU value for the class",
				  "Set/Get the MTU value for the class",
				  0 /* minimum value */,
				  G_MAXUINT /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASS_HTB_MTU,
                                        pspec);

        pspec = g_param_spec_uint("mpu",
				  "The MPU value for the class",
				  "Set/Get the MPU value for the class",
				  0 /* minimum value */,
				  G_MAXUINT /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASS_HTB_MPU,
                                        pspec);
}

GType lql_class_htb_get_type (void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (LQLClassHTBClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        lql_class_htb_class_init,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (LQLClassHTB),
                        0,      /* n_preallocs */
                        lql_class_htb_instance_init    /* instance_init */
                };
                type = g_type_register_static (LQL_TYPE_CLASS,
                                               "LQLClassHTB",
                                               &info, 0);
        }
        return type;
}
