/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include "lql-class-netem.h"
#include "lql-class-netem-private.h"
#include "lql_class_netem_helper.h"

/********************************************************************
 * Non-Virutal API
 ********************************************************************/

/**
 * lql_class_netem_new:
 *
 * Build a new instance of LQLClassNetem. 
 * 
 * Return value: A new LQLClassNetem instance.
 **/
LQLClassNetem*
lql_class_netem_new(void)
{
        return(LQL_CLASS_NETEM(g_object_new(LQL_TYPE_CLASS_NETEM, NULL)));
}

/********************************************************************
 * Virtual API
 ********************************************************************/

/**
 * lql_class_netem_fill_from_nlhdr:
 * @self: The LQLClass object to work with.
 * @hdr: The nl packet to get the queue information from.
 *
 * This method takes a nlmsg from the kernel and set this
 * objects instance variables to match it. This is used
 * to create objects that match the ones currently active
 * in the system. This overrides the default LQLClass
 * implementation of this method.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_class_netem_fill_from_nlhdr(LQLClass *self,
				const struct nlmsghdr *hdr)
{
	g_assert(self != NULL);
	g_assert(hdr != NULL);

	return(lql_class_netem_helper_fill_from_nlhdr(LQL_CLASS_NETEM(self), hdr));
}

/**
 * lql_class_netem_add:
 * @self: The LQLClass object to add to the system.
 * @con: The LQLCon connection to use when adding this queueing discipline.
 *
 * Netem is a classless QDisc. Netem classes cannot be added to the system so
 * the Netem implementation of add just returns FALSE.
 *
 * Returns: FALSE.
 **/
gboolean
lql_class_netem_add(LQLClass *self,
		    LQLCon *con)
{
	g_assert(self != NULL);
	g_assert(con != NULL);

	return(FALSE);
}

/**
 * lql_class_netem_modify:
 * @self: The LQLClass object to modify.
 * @con: The LQLCon connection to use when adding this class.
 *
 * Netem is a classless QDisc. Netem classes cannot be modified. Thus,
 * the Netem implementation of this method just returns FALSE.
 *
 * Returns: FALSE.
 **/
gboolean
lql_class_netem_modify(LQLClass *self,
		       LQLCon *con)
{
	g_assert(self != NULL);
	g_assert(con != NULL);

	return(FALSE);
}

/********************************************************************
 * Object stuff.
 ********************************************************************/
static void
lql_class_netem_instance_init(GTypeInstance *instance,
			      gpointer g_class)
{
	LQLClassNetem *self = (LQLClassNetem *)instance;
	self->private = g_new0(LQLClassNetemPrivate, 1);
	self->private->dispose_has_run = FALSE;

}

static void
lql_class_netem_set_property(GObject *object,
			     guint property_id,
			     const GValue *value,
			     GParamSpec *pspec)
{
        /* LQLClassNetem *self = (LQLClassNetem *) object; */

        switch (property_id) {
        default:
                g_assert(FALSE);
                break;
        }
}

static void
lql_class_netem_get_property(GObject *object,
			     guint property_id,
			     GValue *value,
			     GParamSpec *pspec)
{
        /* LQLClassNetem *self = (LQLClassNetem *) object; */

        switch (property_id) {
        default:
                g_assert(FALSE);
                break;
        }
}

static GObject *
lql_class_netem_constructor(GType type,
			    guint n_construct_properties,
			    GObjectConstructParam *construct_properties)
{
        GObject *obj = NULL;

        {
                /* Invoke parent constructor. */
		LQLClassNetemClass *klass;
		GObjectClass *parent_class;  
		klass = LQL_CLASS_NETEM_CLASS(g_type_class_peek(LQL_TYPE_CLASS_NETEM));
		parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));
		obj = parent_class->constructor(type,
						n_construct_properties,
						construct_properties);
        }
        
        /* do stuff. */

        return obj;
}

static void
lql_class_netem_dispose(GObject *obj)
{
        LQLClassNetem *self = (LQLClassNetem *)obj;

        if (self->private->dispose_has_run) {
                /* If dispose did already run, return. */
                return;
        }
        /* Make sure dispose does not run twice. */
        self->private->dispose_has_run = TRUE;

        /* 
         * In dispose, you are supposed to free all types referenced from this
         * object which might themselves hold a reference to self. Generally,
         * the most simple solution is to unref all members on which you own a 
         * reference.
         */
}

static void
lql_class_netem_finalize(GObject *obj)
{
        LQLClassNetem *self = (LQLClassNetem *)obj;

        /*
         * Here, complete object destruction.
         */
        
        g_free (self->private);
}


static void
lql_class_netem_class_init(gpointer g_class,
			   gpointer g_class_data)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS(g_class);
        LQLClassClass *klass = LQL_CLASS_CLASS(g_class);
        /* GParamSpec *pspec = NULL; */

        gobject_class->set_property = lql_class_netem_set_property;
        gobject_class->get_property = lql_class_netem_get_property;
        gobject_class->dispose = lql_class_netem_dispose;
        gobject_class->finalize = lql_class_netem_finalize;
        gobject_class->constructor = lql_class_netem_constructor;

	/* Set the virtual methods. */
	klass->fill_from_nlhdr = lql_class_netem_fill_from_nlhdr;
	klass->add = lql_class_netem_add;
	klass->modify = lql_class_netem_modify;
	klass->print = lql_class_netem_helper_print;
	klass->printStats = lql_class_netem_helper_printStats;
	klass->updateStats = lql_class_netem_helper_updateStats;

	/* Setup the properties. */
}

GType
lql_class_netem_get_type(void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (LQLClassNetemClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        lql_class_netem_class_init,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (LQLClassNetem),
                        0,      /* n_preallocs */
                        lql_class_netem_instance_init    /* instance_init */
                };
                type = g_type_register_static (LQL_TYPE_CLASS,
                                               "LQLClassNetem",
                                               &info, 0);
        }
        return type;
}
