/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef _LQL_CLASS_H_
#define _LQL_CLASS_H_

#include <glib-object.h>
#include "nl.h"
#include "lql-con.h"
#include "lql-element.h"

#define LQL_TYPE_CLASS (lql_class_get_type ())
#define LQL_CLASS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), LQL_TYPE_CLASS, LQLClass))
#define LQL_CLASS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), LQL_TYPE_CLASS, LQLClassClass))
#define LQL_IS_CLASS(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), LQL_TYPE_CLASS))
#define LQL_IS_CLASS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), LQL_TYPE_CLASS))
#define LQL_CLASS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), LQL_TYPE_CLASS, LQLClassClass))

typedef struct _LQLClass LQLClass;
typedef struct _LQLClassClass LQLClassClass;
typedef struct _LQLClassPrivate LQLClassPrivate;

struct _LQLClass {
	LQLElement parent;

	/* Instance members */
        LQLClassPrivate *private;
};

struct _LQLClassClass {
	LQLElementClass parent;

	/* Class members */
	gboolean (*fill_from_nlhdr) (LQLClass *self, const struct nlmsghdr *hdr);
	gboolean (*add) (LQLClass *self, LQLCon *con);
	gboolean (*modify) (LQLClass *self, LQLCon *con);
	gboolean (*delete) (LQLClass *self, LQLCon *con);
	void (*print) (LQLClass *self);
	void (*printStats) (LQLClass *self);
	gboolean (*updateStats) (LQLClass *class, LQLCon *con);
};

/* used by LQL_TYPE_CLASS */
GType lql_class_get_type (void);

/* Non-virtual API. */
LQLClass* lql_class_new(void);
gboolean lql_class_set_parent(LQLClass *self, LQLElement *element);
gboolean lql_class_set_id(LQLClass *self, guint16 id);
guint16 lql_class_get_id(LQLClass *self);
unsigned int lql_class_get_priority(LQLClass *self);
gboolean lql_class_set_priority(LQLClass *self, unsigned int priority);

guint64 lql_class_get_bytes(LQLClass *self);
guint32 lql_class_get_packets(LQLClass *self);
guint32 lql_class_get_drops(LQLClass *self);
guint32 lql_class_get_overlimits(LQLClass *self);
guint32 lql_class_get_bps(LQLClass *self);
guint32 lql_class_get_pps(LQLClass *self);
guint32 lql_class_get_qlen(LQLClass *self);
guint32 lql_class_get_backlog(LQLClass *self);

/* guint32 lql_class_get_id(LQLClass *self); */
/* gboolean lql_class_set_id(LQLClass *self, guint32 id); */
/* int lql_class_get_ifIndex(LQLClass *self); */
/* gboolean lql_class_set_ifIndex(LQLClass *self, int ifIndex); */
/* unsigned int lql_class_get_priority(LQLClass *self); */
/* gboolean lql_class_set_priority(LQLClass *self, unsigned int priority); */
/* gboolean lql_class_set_handle(LQLClass *self, guint16 handle); */
/* guint16 lql_class_get_handle(LQLClass *self); */
/* gboolean lql_class_set_parent(LQLClass *self, guint32 parent); */
/* guint32 lql_class_get_parent(LQLClass *self); */

/* Virtual API. */
gboolean lql_class_fill_from_nlhdr(LQLClass *self, const struct nlmsghdr *hdr);
gboolean lql_class_add(LQLClass *self, LQLCon *con);
gboolean lql_class_modify(LQLClass *self, LQLCon *con);
gboolean lql_class_delete(LQLClass *self, LQLCon *con);
void lql_class_print(LQLClass *self);
void lql_class_printStats(LQLClass *self);
gboolean lql_class_updateStats(LQLClass *self, LQLCon *con);

#endif /* _LQL_CLASS_H_ */
