/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include "lql-classifier-tcindex.h"
#include "lql-classifier-tcindex-private.h"
#include "lql_classifier_tcindex_helper.h"

/********************************************************************
 * Non-Virutal API
 ********************************************************************/

/**
 * lql_classifier_tcindex_new:
 *
 * Build a new instance of LQLClassifierTCIndex. 
 * 
 * Return value: A new LQLClassifierTCIndex instance.
 **/
LQLClassifierTCIndex*
lql_classifier_tcindex_new(void)
{
        return(LQL_CLASSIFIER_TCINDEX(g_object_new(LQL_TYPE_CLASSIFIER_TCINDEX, NULL)));
}


guint32
lql_classifier_tcindex_get_hash(LQLClassifierTCIndex *self)
{
        guint32 mark = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(0);
        }
        
        g_object_get(LQL_CLASSIFIER_TCINDEX(self), "hash", &mark, NULL);

        return(mark);
}


gboolean
lql_classifier_tcindex_set_hash(LQLClassifierTCIndex *self,
				guint32 hash)
{
        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASSIFIER_TCINDEX(self), "hash", hash, NULL);
        
        return(TRUE);
}

guint32
lql_classifier_tcindex_get_mask(LQLClassifierTCIndex *self)
{
        guint32 mask = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(0);
        }
        
        g_object_get(LQL_CLASSIFIER_TCINDEX(self), "mask", &mask, NULL);

        return(mask);
}


gboolean
lql_classifier_tcindex_set_mask(LQLClassifierTCIndex *self,
				guint32 mask)
{
        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASSIFIER_TCINDEX(self), "mask", mask, NULL);
        
        return(TRUE);
}

guint32
lql_classifier_tcindex_get_shift(LQLClassifierTCIndex *self)
{
        guint32 shift = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(0);
        }
        
        g_object_get(LQL_CLASSIFIER_TCINDEX(self), "shift", &shift, NULL);

        return(shift);
}


gboolean
lql_classifier_tcindex_set_shift(LQLClassifierTCIndex *self,
				guint32 shift)
{
        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASSIFIER_TCINDEX(self), "shift", shift, NULL);
        
        return(TRUE);
}

gboolean
lql_classifier_tcindex_get_fallThrough(LQLClassifierTCIndex *self)
{
        guint32 fall = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(0);
        }
        
        g_object_get(LQL_CLASSIFIER_TCINDEX(self), "fall-through", &fall, NULL);

        return(fall);
}


gboolean
lql_classifier_tcindex_set_fallThrough(LQLClassifierTCIndex *self,
				       gboolean fall)
{
        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASSIFIER_TCINDEX(self), "fall-through", fall, NULL);
        
        return(TRUE);
}

guint32
lql_classifier_tcindex_get_classHandle(LQLClassifierTCIndex *self)
{
        guint32 class = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(0);
        }
        
        g_object_get(LQL_CLASSIFIER_TCINDEX(self), "class", &class, NULL);

        return(class);
}


gboolean
lql_classifier_tcindex_set_class(LQLClassifierTCIndex *self,
				 LQLClass *class)
{
        g_assert(self != NULL);
	g_assert(class != NULL);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASSIFIER_TCINDEX(self), "fall-through",
		     lql_element_get_handle(LQL_ELEMENT(class)), NULL);
        
        return(TRUE);
}

/********************************************************************
 * Virtual API
 ********************************************************************/

/**
 * lql_classifier_tcindex_fill_from_nlhdr:
 * @self: The LQLClassifier object to work with.
 * @hdr: The nl packet to get the queue information from.
 *
 * This method takes a nlmsg from the kernel and set this
 * objects instance variables to match it. This is used
 * to create objects that match the ones currently active
 * in the system. This overrides the default LQLClassifier
 * implementation of this method.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_classifier_tcindex_fill_from_nlhdr(LQLClassifier *self,
				      const struct nlmsghdr *hdr)
{
	g_assert(self != NULL);
	g_assert(hdr != NULL);

	return(lql_classifier_tcindex_helper_fill_from_nlhdr(LQL_CLASSIFIER_TCINDEX(self), hdr));
}

/**
 * lql_classifier_tcindex_add:
 * @self: The LQLClassifier object to add to the system.
 * @con: The LQLCon connection to use when adding this queueing discipline.
 *
 * Add this HTB queueing discipline to the system. This overrides
 * the default implementation of this function in LQLClassifier.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_classifier_tcindex_add(LQLClassifier *self,
			  LQLCon *con)
{
	g_assert(self != NULL);
	g_assert(con != NULL);

	return(lql_classifier_tcindex_helper_add(LQL_CLASSIFIER_TCINDEX(self), con));
}

/**
 * lql_classifier_tcindex_modify:
 * @self: The LQLClassifier object to modify.
 * @con: The LQLCon connection to use when adding this classifier.
 *
 * Modify the HTB queueing discipline in the system to match the current
 * object state. This method overrides the default
 * implementation in LQLClassifier.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_classifier_tcindex_modify(LQLClassifier *self,
			     LQLCon *con)
{
	g_assert(self != NULL);
	g_assert(con != NULL);

	return(lql_classifier_tcindex_helper_modify(LQL_CLASSIFIER_TCINDEX(self), con));
}

/********************************************************************
 * Object stuff.
 ********************************************************************/
enum {
	LQL_CLASSIFIER_TCINDEX_HASH = 1,
	LQL_CLASSIFIER_TCINDEX_MASK = 2,
	LQL_CLASSIFIER_TCINDEX_SHIFT = 3,
	LQL_CLASSIFIER_TCINDEX_FALL_THROUGH = 4,
	LQL_CLASSIFIER_TCINDEX_CLASS = 5,
};

static void
lql_classifier_tcindex_instance_init(GTypeInstance *instance,
				    gpointer g_class)
{
	LQLClassifierTCIndex *self = (LQLClassifierTCIndex *)instance;
	self->private = g_new0(LQLClassifierTCIndexPrivate, 1);
	self->private->dispose_has_run = FALSE;
}

static void
lql_classifier_tcindex_set_property(GObject *object,
				    guint property_id,
				    const GValue *value,
				    GParamSpec *pspec)
{
        LQLClassifierTCIndex *self = (LQLClassifierTCIndex *) object;

        switch (property_id) {
        case LQL_CLASSIFIER_TCINDEX_HASH:
        {
                self->private->hash = g_value_get_uint(value);
                break;
        }
        case LQL_CLASSIFIER_TCINDEX_MASK:
        {
                self->private->mask = g_value_get_uint(value);
                break;
        }
        case LQL_CLASSIFIER_TCINDEX_SHIFT:
        {
                self->private->shift = g_value_get_uint(value);
                break;
        }
	case LQL_CLASSIFIER_TCINDEX_FALL_THROUGH:
	{
		self->private->fallThrough = g_value_get_boolean(value);
		break;
	}
        case LQL_CLASSIFIER_TCINDEX_CLASS:
        {
                self->private->class = g_value_get_uint(value);
                break;
        }
        default:
                g_assert(FALSE);
                break;
        }
}

static void
lql_classifier_tcindex_get_property(GObject *object,
				    guint property_id,
				    GValue *value,
				    GParamSpec *pspec)
{
        LQLClassifierTCIndex *self = (LQLClassifierTCIndex *) object;

        switch (property_id) {
        case LQL_CLASSIFIER_TCINDEX_HASH:
        {
                g_value_set_uint(value, self->private->hash);
                break;
        }
        case LQL_CLASSIFIER_TCINDEX_MASK:
        {
                g_value_set_uint(value, self->private->mask);
                break;
        }
        case LQL_CLASSIFIER_TCINDEX_SHIFT:
        {
                g_value_set_uint(value, self->private->shift);
                break;
        }
	case LQL_CLASSIFIER_TCINDEX_FALL_THROUGH:
	{
		g_value_set_boolean(value, self->private->fallThrough);
		break;
	}
        case LQL_CLASSIFIER_TCINDEX_CLASS:
        {
                g_value_set_uint(value, self->private->class);
                break;
        }
        default:
                g_assert(FALSE);
                break;
        }
}

static GObject *
lql_classifier_tcindex_constructor(GType type,
				  guint n_construct_properties,
				  GObjectConstructParam *construct_properties)
{
        GObject *obj = NULL;

        {
                /* Invoke parent constructor. */
		LQLClassifierTCIndexClass *klass;
		GObjectClass *parent_class;  
		klass = LQL_CLASSIFIER_TCINDEX_CLASS(g_type_class_peek(LQL_TYPE_CLASSIFIER_TCINDEX));
		parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));
		obj = parent_class->constructor(type,
						n_construct_properties,
						construct_properties);
        }
        
        /* do stuff. */

        return obj;
}

static void
lql_classifier_tcindex_dispose(GObject *obj)
{
        LQLClassifierTCIndex *self = (LQLClassifierTCIndex *)obj;

        if (self->private->dispose_has_run) {
                /* If dispose did already run, return. */
                return;
        }
        /* Make sure dispose does not run twice. */
        self->private->dispose_has_run = TRUE;

        /* 
         * In dispose, you are supposed to free all types referenced from this
         * object which might themselves hold a reference to self. Generally,
         * the most simple solution is to unref all members on which you own a 
         * reference.
         */
}

static void
lql_classifier_tcindex_finalize(GObject *obj)
{
        LQLClassifierTCIndex *self = (LQLClassifierTCIndex *)obj;

        /*
         * Here, complete object destruction.
         */
        
        g_free (self->private);
}


static void
lql_classifier_tcindex_class_init(gpointer g_class,
				 gpointer g_class_data)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS(g_class);
        LQLClassifierClass *klass = LQL_CLASSIFIER_CLASS(g_class);
        GParamSpec *pspec = NULL;

        gobject_class->set_property = lql_classifier_tcindex_set_property;
        gobject_class->get_property = lql_classifier_tcindex_get_property;
        gobject_class->dispose = lql_classifier_tcindex_dispose;
        gobject_class->finalize = lql_classifier_tcindex_finalize;
        gobject_class->constructor = lql_classifier_tcindex_constructor;

	/* Set the virtual methods. */
	klass->fill_from_nlhdr = lql_classifier_tcindex_fill_from_nlhdr;
	klass->add = lql_classifier_tcindex_add;
	klass->modify = lql_classifier_tcindex_modify;

	/* Setup the properties. */
        pspec = g_param_spec_uint("hash",
				  ".",
				  "Set/Get.",
				  0 /* minimum value */,
				  0x10000 /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASSIFIER_TCINDEX_HASH,
                                        pspec);

        pspec = g_param_spec_uint("mask",
				  ".",
				  "Set/Get.",
				  0 /* minimum value */,
				  0x10000 /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASSIFIER_TCINDEX_MASK,
                                        pspec);

        pspec = g_param_spec_uint("shift",
				  ".",
				  "Set/Get.",
				  0 /* minimum value */,
				  G_MAXUINT /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASSIFIER_TCINDEX_SHIFT,
                                        pspec);

        pspec = g_param_spec_boolean("fall-through",
				  ".",
				  "Set/Get.",
				  FALSE /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASSIFIER_TCINDEX_FALL_THROUGH,
                                        pspec);

        pspec = g_param_spec_uint("class",
				  ".",
				  "Set/Get.",
				  0 /* minimum value */,
				  G_MAXUINT /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASSIFIER_TCINDEX_CLASS,
                                        pspec);
}

GType
lql_classifier_tcindex_get_type(void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (LQLClassifierTCIndexClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        lql_classifier_tcindex_class_init,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (LQLClassifierTCIndex),
                        0,      /* n_preallocs */
                        lql_classifier_tcindex_instance_init    /* instance_init */
                };
                type = g_type_register_static (LQL_TYPE_CLASSIFIER,
                                               "LQLClassifierTCIndex",
                                               &info, 0);
        }
        return type;
}
