/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef _LQL_CLASSIFIER_H_
#define _LQL_CLASSIFIER_H_

#include <glib-object.h>

#include "nl.h"
#include "lql-con.h"
#include "lql-element.h"
#include "lql-interface.h"
#include "lql-class.h"

#define LQL_TYPE_CLASSIFIER (lql_classifier_get_type ())
#define LQL_CLASSIFIER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), LQL_TYPE_CLASSIFIER, LQLClassifier))
#define LQL_CLASSIFIER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), LQL_TYPE_CLASSIFIER, LQLClassifierClass))
#define LQL_IS_CLASSIFIER(obj) (G_TYPE_CHECK_TYPE ((obj), LQL_TYPE_CLASSIFIER))
#define LQL_IS_CLASSIFIER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), LQL_TYPE_CLASSIFIER))
#define LQL_CLASSIFIER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), LQL_TYPE_CLASSIFIER, LQLClassifierClass))

typedef struct _LQLClassifier LQLClassifier;
typedef struct _LQLClassifierClass LQLClassifierClass;
typedef struct _LQLClassifierPrivate LQLClassifierPrivate;

struct _LQLClassifier {
	GObject parent;

	/* Instance members */
        LQLClassifierPrivate *private;
};

struct _LQLClassifierClass {
	GObjectClass parent;

	/* Class members */
	gboolean (*fill_from_nlhdr) (LQLClassifier *self, const struct nlmsghdr *hdr);
	gboolean (*add) (LQLClassifier *self, LQLCon *con);
	gboolean (*modify) (LQLClassifier *self, LQLCon *con);
	gboolean (*delete) (LQLClassifier *self, LQLCon *con);
};

/* used by LQL_TYPE_CLASSIFIER */
GType lql_classifier_get_type (void);

/* Non-virtual API. */
LQLClassifier* lql_classifier_new(void);

gboolean lql_classifier_set_interface(LQLClassifier *self, LQLInterface *interface);
int lql_classifier_get_ifIndex(LQLClassifier *self);
gboolean lql_classifier_set_ifIndex(LQLClassifier *self, int ifIndex);

gboolean lql_classifier_set_parent(LQLClassifier *self, LQLElement *parent);
guint32 lql_classifier_get_parentHandle(LQLClassifier *self);
gboolean lql_classifier_set_parentHandle(LQLClassifier *self, guint32 parent);

guint32 lql_classifier_get_id(LQLClassifier *self);
gboolean lql_classifier_set_id(LQLClassifier *self, guint32 id);

gboolean lql_classifier_set_class(LQLClassifier *self, LQLClass *class);
guint32 lql_classifier_get_classHandle(LQLClassifier *self);
gboolean lql_classifier_set_classHandle(LQLClassifier *self, guint32 class);

unsigned int lql_classifier_get_priority(LQLClassifier *self);
gboolean lql_classifier_set_priority(LQLClassifier *self, unsigned int priority);

unsigned int lql_classifier_get_protocol(LQLClassifier *self);
gboolean lql_classifier_set_protocol(LQLClassifier *self, unsigned int protocol);

/* Virtual API. */
gboolean lql_classifier_fill_from_nlhdr(LQLClassifier *self, const struct nlmsghdr *hdr);
gboolean lql_classifier_add(LQLClassifier *self, LQLCon *con);
gboolean lql_classifier_modify(LQLClassifier *self, LQLCon *con);
gboolean lql_classifier_delete(LQLClassifier *self, LQLCon *con);

#endif /* _LQL_CLASSIFIER_H_ */
