/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include "lql-u32match.h"
#include "lql-u32match-private.h"

/********************************************************************
 * Non-Virtual API
 ********************************************************************/

/**
 * lql_u32match_new:
 *
 * Build a new instance of LQLU32Match. 
 * 
 * Return value: A new LQLU32Match instance.
 **/
LQLU32Match*
lql_u32match_new(void)
{
        return(LQL_U32MATCH(g_object_new(LQL_TYPE_U32MATCH, NULL)));
}

guint32
lql_u32match_get_value(LQLU32Match *self)
{
        guint32 value = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(-1);
        }
        
        g_object_get(LQL_U32MATCH(self), "value", &value, NULL);

        return(value);
}

gboolean
lql_u32match_set_value(LQLU32Match *self,
		       guint32 value)
{
        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_U32MATCH(self), "value", value, NULL);
        
        return(TRUE);
}

guint32
lql_u32match_get_mask(LQLU32Match *self)
{
        guint32 mask = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(-1);
        }
        
        g_object_get(LQL_U32MATCH(self), "mask", &mask, NULL);

        return(mask);
}

gboolean
lql_u32match_set_mask(LQLU32Match *self,
		      guint32 mask)
{
        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_U32MATCH(self), "mask", mask, NULL);
        
        return(TRUE);
}

int
lql_u32match_get_off(LQLU32Match *self)
{
        int off = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(-1);
        }

        g_object_get(LQL_U32MATCH(self), "off", &off, NULL);

        return(off);
}

gboolean
lql_u32match_set_off(LQLU32Match *self,
		     int off)
{
        g_assert(self != NULL);
        g_assert(off > 0);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_U32MATCH(self), "off", off, NULL);
        
        return(TRUE);
}

int
lql_u32match_get_offset(LQLU32Match *self)
{
        int offset = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(-1);
        }

        g_object_get(LQL_U32MATCH(self), "offset", &offset, NULL);

        return(offset);
}

gboolean
lql_u32match_set_offset(LQLU32Match *self,
		     int offset)
{
        g_assert(self != NULL);
        g_assert(offset > 0);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_U32MATCH(self), "offset", offset, NULL);
        
        return(TRUE);
}


/********************************************************************
 * Virtual API
 ********************************************************************/

/********************************************************************
 * Object stuff.
 ********************************************************************/
enum {
        LQL_U32MATCH_VALUE = 1,
	LQL_U32MATCH_MASK = 2,
	LQL_U32MATCH_OFF = 3,
	LQL_U32MATCH_OFFSET = 4,
};

static void
lql_u32match_instance_init(GTypeInstance *instance,
			gpointer g_class)
{
        LQLU32Match *self = (LQLU32Match *)instance;
        self->private = g_new0 (LQLU32MatchPrivate, 1);
        self->private->dispose_has_run = FALSE;
}

static void
lql_u32match_set_property(GObject *object,
		       guint property_id,
		       const GValue *value,
		       GParamSpec *pspec)
{
        LQLU32Match *self = (LQLU32Match *) object;

        switch (property_id) {
	case LQL_U32MATCH_VALUE:
	{
		self->private->value = g_value_get_uint(value);
		break;
	}
	case LQL_U32MATCH_MASK:
	{
		self->private->mask = g_value_get_uint(value);
		break;
	}
        case LQL_U32MATCH_OFF:
        {
                self->private->off = g_value_get_int(value);
                break;
        }
        case LQL_U32MATCH_OFFSET:
        {
                self->private->offset = g_value_get_int(value);
                break;
        }
        default:
                g_assert (FALSE);
                break;
        }
}

static void
lql_u32match_get_property(GObject *object,
		       guint property_id,
		       GValue *value,
		       GParamSpec *pspec)
{
        LQLU32Match *self = (LQLU32Match *) object;

        switch (property_id) {
	case LQL_U32MATCH_VALUE:
	{
		g_value_set_uint(value, self->private->value);
		break;
	}
	case LQL_U32MATCH_MASK:
	{
		g_value_set_uint(value, self->private->mask);
		break;
	}
        case LQL_U32MATCH_OFF:
        {
                g_value_set_int(value, self->private->off);
                break;
        }
        case LQL_U32MATCH_OFFSET:
        {
                g_value_set_int(value, self->private->offset);
                break;
        }
        default:
                g_assert (FALSE);
                break;
        }
}

static GObject *
lql_u32match_constructor(GType type,
		      guint n_construct_properties,
		      GObjectConstructParam *construct_properties)
{
        GObject *obj;

        {
                /* Invoke parent constructor. */
                LQLU32MatchClass *klass;
                GObjectClass *parent_class;  
                klass = LQL_U32MATCH_CLASS(g_type_class_peek(LQL_TYPE_U32MATCH));
                parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));
                obj = parent_class->constructor(type,
						n_construct_properties,
						construct_properties);
        }
        
        /* do stuff. */

        return obj;
}

static void
lql_u32match_dispose (GObject *obj)
{
        LQLU32Match *self = (LQLU32Match *)obj;

        if (self->private->dispose_has_run) {
                /* If dispose did already run, return. */
                return;
        }
        /* Make sure dispose does not run twice. */
        self->private->dispose_has_run = TRUE;

        /* 
         * In dispose, you are supposed to free all types referenced from this
         * object which might themselves hold a reference to self. Generally,
         * the most simple solution is to unref all members on which you own a 
         * reference.
         */
}

static void
lql_u32match_finalize (GObject *obj)
{
        LQLU32Match *self = (LQLU32Match *)obj;

        /*
         * Here, complete object destruction.
         * You might not need to do much...
         */

        
        g_free (self->private);
}

static void
lql_u32match_class_init (gpointer g_class,
		      gpointer g_class_data)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS(g_class);
        //LQLU32MatchClass *klass = LQL_U32MATCH_CLASS(g_class);
        GParamSpec *pspec = NULL;

        gobject_class->set_property = lql_u32match_set_property;
        gobject_class->get_property = lql_u32match_get_property;
        gobject_class->dispose = lql_u32match_dispose;
        gobject_class->finalize = lql_u32match_finalize;
        gobject_class->constructor = lql_u32match_constructor;

	/* Set the virtual methods. */

	/* Setup the properties. */
        pspec = g_param_spec_uint("value",
				  "value",
				  "Set/Get the value",
				  0 /* minimum value */,
				  G_MAXUINT32 /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_U32MATCH_VALUE,
                                        pspec);

        pspec = g_param_spec_uint("mask",
				  "mask",
				  "Set/Get the mask",
				  0 /* minimum value */,
				  G_MAXUINT32 /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_U32MATCH_MASK,
                                        pspec);


        pspec = g_param_spec_int("off",
                                 "off",
                                 "Set/Get the off"
				 " attached to",
                                 G_MININT /* minimum value */,
                                 G_MAXINT /* maximum value */,
                                 0 /* default value */,
                                 G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_U32MATCH_OFF,
                                        pspec);

        pspec = g_param_spec_int("offset",
                                 "offset",
                                 "Set/Get the offset"
				 " attached to",
                                 G_MININT /* minimum value */,
                                 G_MAXINT /* maximum value */,
                                 0 /* default value */,
                                 G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_U32MATCH_OFFSET,
                                        pspec);
}

GType lql_u32match_get_type (void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (LQLU32MatchClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        lql_u32match_class_init,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (LQLU32Match),
                        0,      /* n_preallocs */
                        lql_u32match_instance_init    /* instance_init */
                };
                type = g_type_register_static(G_TYPE_OBJECT,
					      "LQLU32Match",
					      &info, 0);
        }

        return type;
}
