/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <unistd.h>
#include <string.h>

#include "debug_macros.h"
#include "nl.h"
#include "lql-con.h"
#include "lql-interface.h"
#include "lql-class-netem.h"
#include "lql-class-netem-private.h"

gboolean
lql_class_netem_helper_fill_from_nlhdr(LQLClassNetem *netem,
				       const struct nlmsghdr *hdr)
{
	g_assert(netem != NULL);
	g_assert(hdr != NULL);

	lql_element_set_handle(LQL_ELEMENT(netem), nl_tcmsg_get_handle(hdr));
	lql_element_set_parentHandle(LQL_ELEMENT(netem), nl_tcmsg_get_parent(hdr));
	lql_element_set_ifIndex(LQL_ELEMENT(netem), nl_tcmsg_get_ifindex(hdr));

	return(TRUE);
}

void
lql_class_netem_helper_print(LQLClass *self)
{
	LQLClassNetemClass *klass = NULL;
	LQLClassClass *parentKlass = NULL;
	LQLClassNetem *netem = NULL;

	g_assert(self != NULL);
	g_assert(LQL_IS_CLASS_NETEM(self));

	netem = LQL_CLASS_NETEM(self);

        if (netem->private->dispose_has_run) {
                return;
        }

	klass = LQL_CLASS_NETEM_GET_CLASS(self);
	parentKlass = g_type_class_peek_parent(klass);
	
	parentKlass->print(self);

	g_print("Type: Netem\n");
}

void
lql_class_netem_helper_printStats(LQLClass *self)
{
	LQLClassNetemClass *klass = NULL;
	LQLClassClass *parentKlass = NULL;
	LQLClassNetem *netem = NULL;

	g_assert(self != NULL);
	g_assert(LQL_IS_CLASS_NETEM(self));

	netem = LQL_CLASS_NETEM(self);

        if (netem->private->dispose_has_run) {
                return;
        }

	klass = LQL_CLASS_NETEM_GET_CLASS(self);
	parentKlass = g_type_class_peek_parent(klass);
	
	parentKlass->printStats(self);
}

gboolean
lql_class_netem_helper_updateStats(LQLClass *self,
				   LQLCon *con)
{
	LQLClassNetemClass *klass = NULL;
	LQLClassClass *parentKlass = NULL;
	LQLClassNetem *netem = NULL;

	g_assert(self != NULL);
	g_assert(LQL_IS_CLASS_NETEM(self));

	netem = LQL_CLASS_NETEM(self);

        if (netem->private->dispose_has_run) {
                return(FALSE);
        }

	klass = LQL_CLASS_NETEM_GET_CLASS(self);
	parentKlass = g_type_class_peek_parent(klass);
	
	return(parentKlass->updateStats(self, con));
}
