/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <unistd.h>
#include <string.h>

#include "debug_macros.h"
#include "nl.h"
#include "lql-con.h"
#include "lql-interface.h"
#include "lql-class-priomap.h"
#include "lql-class-priomap-private.h"

gboolean
lql_class_priomap_helper_fill_from_nlhdr(LQLClassPriomap *class,
					 const struct nlmsghdr *hdr)
{
	g_assert(class != NULL);
	g_assert(hdr != NULL);

	lql_element_set_handle(LQL_ELEMENT(class), nl_tcmsg_get_handle(hdr));
	lql_element_set_parentHandle(LQL_ELEMENT(class), nl_tcmsg_get_parent(hdr));
	lql_element_set_ifIndex(LQL_ELEMENT(class), nl_tcmsg_get_ifindex(hdr));

	return(TRUE);
}

void
lql_class_priomap_helper_print(LQLClass *self)
{
	LQLClassPriomapClass *klass = NULL;
	LQLClassClass *parentKlass = NULL;
	LQLClassPriomap *priomap = NULL;

	g_assert(self != NULL);
	g_assert(LQL_IS_CLASS_PRIOMAP(self));

	priomap = LQL_CLASS_PRIOMAP(self);

        if (priomap->private->dispose_has_run) {
                return;
        }

	klass = LQL_CLASS_PRIOMAP_GET_CLASS(self);
	parentKlass = g_type_class_peek_parent(klass);
	
	parentKlass->print(self);

	g_print("Type: Priomap\n");
}

void
lql_class_priomap_helper_printStats(LQLClass *self)
{
	g_assert(self != NULL);
	g_assert(LQL_IS_CLASS_PRIOMAP(self));
	
	g_print("Priomap does not support class statistics.\n");
}
