/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <unistd.h>
#include <glib.h>

#include "lql-classifier.h"
#include "lql-classifier-private.h"

gboolean
lql_classifier_helper_delete(LQLClassifier *classifier,
			     LQLCon *con)
{
	TcPacket *pkt = NULL;
	struct nlmsghdr *nlmsg = NULL;

	g_assert(classifier != NULL);	
	g_assert(con != NULL);

	pkt = nl_tcpacket_new();
	nlmsg = nl_tcpacket_get_nlmsg(pkt);

	nl_nlmsg_set_type(nlmsg, RTM_DELTFILTER);
	nl_nlmsg_set_flags(nlmsg, NLM_F_REQUEST | NLM_F_ACK);
	nl_nlmsg_set_pid(nlmsg, getpid());
	nl_nlmsg_set_seq(nlmsg, lql_con_get_inc_sequence(LQL_CON(con)));

	nl_tcmsg_set_family(nlmsg, AF_UNSPEC);
	nl_tcmsg_set_ifindex(nlmsg, lql_classifier_get_ifIndex(LQL_CLASSIFIER(classifier)));
	nl_tcmsg_set_parent(nlmsg,
			    lql_classifier_get_parentHandle(LQL_CLASSIFIER(classifier)));
	/* At least for TCIndex, this cannot be set when deleting a filter. */
	/* Leaving the line here because I have an odd feeling this will cause trouble
	   with other classifiers later. */
	/* It may be best to just not set the handle in tcindexs fill_nlhdr(). */
	// nl_tcmsg_set_handle(nlmsg, lql_classifier_get_id(LQL_CLASSIFIER(classifier)));
	nl_tcmsg_set_info(nlmsg,
			  TC_H_MAKE(lql_classifier_get_priority(LQL_CLASSIFIER(classifier))<<16,
				    lql_classifier_get_protocol(LQL_CLASSIFIER(classifier))));

	if (!nl_tcpacket_do_command(pkt, lql_con_get_nlcon(LQL_CON(con)))) {
		g_printerr("lql_classifier_helper_delete: Could not do command.\n");
		return(FALSE);
	}

	return(TRUE);
}
