/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <unistd.h>
#include <string.h>
#include <math.h>

#include "debug_macros.h"
#include "nl.h"
#include "lql-con.h"
#include "lql-qdisc.h"
#include "lql-qdisc-netem.h"
#include "lql-qdisc-netem-private.h"

#define US_IN_MS 1000

static guint32 lql_qdisc_netem_helper_per_to_u32(float percent);

// Horrible hack. Fix.
extern double tick;

gboolean
lql_qdisc_netem_helper_fill_from_nlhdr(LQLQDiscNetem *qdisc,
				       const struct nlmsghdr *hdr)
{
	struct tc_netem_qopt *opt = NULL;
	struct tc_netem_corr *cor = NULL;
	guint32 tmp = 0;
	float tmp2 = 0;

	g_assert(qdisc != NULL);
	g_assert(hdr != NULL);

	lql_element_fill_from_nlhdr(LQL_ELEMENT(qdisc), hdr);
	
	opt = (struct tc_netem_qopt *)nl_tcmsg_get_rta_val(hdr, TCA_OPTIONS);
	if (opt == NULL) {
		g_printerr("Could not get netem options.\n");
		return(FALSE);
	}

	tmp = rint((opt->latency / tick) / US_IN_MS);
	lql_qdisc_netem_set_latency(LQL_QDISC_NETEM(qdisc), tmp);

	lql_qdisc_netem_set_limit(LQL_QDISC_NETEM(qdisc), opt->limit);

	tmp2 = 100.0 * ((float) opt->loss / 0xffffffff);
	lql_qdisc_netem_set_loss(LQL_QDISC_NETEM(qdisc), tmp2);

	lql_qdisc_netem_set_gap(LQL_QDISC_NETEM(qdisc), opt->gap);

 	tmp2 = 100.0 * ((float) opt->duplicate / 0xffffffff);
	lql_qdisc_netem_set_duplicate(LQL_QDISC_NETEM(qdisc), tmp2);

	tmp = rint((opt->jitter / tick) / US_IN_MS);
	lql_qdisc_netem_set_jitter(LQL_QDISC_NETEM(qdisc), tmp);

	/* Normally, nl_tcmsg_get_rta_sub_val() would be used to get a sub RTA-type.
	   However, in this situation, the netem QDisc does something that the other QDiscs
	   don't; it puts its tc_netem_qopt struct in TCA_OPTIONS then packs another
	   rtval directly under TCA_OPTIONS. The other QDiscs make TCA_OPTIONS a 0 length
	   attribute then put sub-attributes under TCA_OPTIONS. Since there is no nl_ API
	   to handle the way netem does it we do some weird pointer arithmetic and
	   casting below. */
	cor = (struct tc_netem_corr *)((char *)opt + sizeof(struct rtattr) +
				       sizeof(struct tc_netem_qopt));

	tmp2 = 100.0 * (double) cor->delay_corr / 0xffffffff;
	lql_qdisc_netem_set_latencyCorrelation(qdisc, tmp2);

 	tmp2 = 100.0 * (double) cor->loss_corr / 0xffffffff;
	lql_qdisc_netem_set_lossCorrelation(qdisc, tmp2);
	
	tmp2 = 100.0 * (double) cor->dup_corr / 0xffffffff;
	lql_qdisc_netem_set_duplicateCorrelation(qdisc, tmp2);

	return(TRUE);
}

static gboolean
lql_qdisc_netem_helper_do(LQLQDiscNetem *qdisc,
			  LQLCon *con,
			  int flags)
{
	TcPacket *pkt = NULL;
	struct nlmsghdr *hdr = NULL;
	struct tc_netem_qopt *opt = NULL;
	struct tc_netem_corr *cor = NULL;
	gchar *kind = "netem";

	g_assert(con != NULL);
	g_assert(LQL_IS_CON(con));
	g_assert(qdisc != NULL);
	g_assert(LQL_IS_QDISC_NETEM(qdisc));

	pkt = nl_tcpacket_new();
	hdr = nl_tcpacket_get_nlmsg(pkt);

	nl_nlmsg_set_type(hdr, RTM_NEWQDISC);
	nl_nlmsg_set_flags(hdr, NLM_F_REQUEST | NLM_F_ACK | flags);
	nl_nlmsg_set_pid(hdr, getpid());
	nl_nlmsg_set_seq(hdr, lql_con_get_inc_sequence(LQL_CON(con)));

	nl_tcmsg_set_family(hdr, AF_UNSPEC);
	nl_tcmsg_set_ifindex(hdr, lql_element_get_ifIndex(LQL_ELEMENT(qdisc)));
	nl_tcmsg_set_parent(hdr, lql_element_get_parentHandle(LQL_ELEMENT(qdisc)));
	nl_tcmsg_set_handle(hdr, lql_element_get_handle(LQL_ELEMENT(qdisc)));

	opt = g_malloc0(sizeof(struct tc_netem_qopt));
	opt->latency = (lql_qdisc_netem_get_latency(qdisc) * US_IN_MS) *
		lql_con_get_tickInUsec(con);
	opt->limit = lql_qdisc_netem_get_limit(qdisc);
	opt->loss = lql_qdisc_netem_helper_per_to_u32(lql_qdisc_netem_get_loss(qdisc));
	opt->gap = lql_qdisc_netem_get_gap(qdisc);
	opt->duplicate = lql_qdisc_netem_helper_per_to_u32(
		lql_qdisc_netem_get_duplicate(qdisc));
	opt->jitter = (lql_qdisc_netem_get_jitter(qdisc) * US_IN_MS) *
		lql_con_get_tickInUsec(con);

	cor = g_malloc0(sizeof(struct tc_netem_corr));
	cor->delay_corr = lql_qdisc_netem_helper_per_to_u32(
		lql_qdisc_netem_get_latencyCorrelation(qdisc));
	cor->loss_corr = lql_qdisc_netem_helper_per_to_u32(
		lql_qdisc_netem_get_lossCorrelation(qdisc));
	cor->dup_corr = lql_qdisc_netem_helper_per_to_u32(
		lql_qdisc_netem_get_duplicateCorrelation(qdisc));

	if (!nl_tcpacket_add_rtattr(pkt, TCA_KIND, strlen(kind) + 1, kind)) {
		debug_print(5, "Could not add TCA_KIND");
	}
	if (!nl_tcpacket_add_rtattr(pkt, TCA_OPTIONS, sizeof(struct tc_netem_qopt), opt)) {
		debug_print(5, "Could not add TCA_OPTIONS");
	}
	if (!nl_tcpacket_add_rtattr_sub(pkt, TCA_OPTIONS, TCA_NETEM_CORR,
					sizeof(struct tc_netem_corr), cor)) {
		debug_print(5, "Could not add TCA_NETEM_CORR");
	}

	if (!nl_tcpacket_do_command(pkt, lql_con_get_nlcon(LQL_CON(con)))) {
		return(FALSE);
	}

	g_free(opt);
	g_free(cor);
	g_free(pkt);

	return(TRUE);
}

gboolean
lql_qdisc_netem_helper_add(LQLQDiscNetem *qdisc,
			   LQLCon *con)
{
	g_assert(qdisc != NULL);
	g_assert(con != NULL);

	return(lql_qdisc_netem_helper_do(qdisc, con, NLM_F_CREATE | NLM_F_EXCL));
}

gboolean
lql_qdisc_netem_helper_modify(LQLQDiscNetem *qdisc,
			      LQLCon *con)
{
	g_assert(qdisc != NULL);
	g_assert(con != NULL);

	return(lql_qdisc_netem_helper_do(qdisc, con, 0));
}

void
lql_qdisc_netem_helper_print(LQLQDisc *self)
{
	LQLQDiscNetemClass *klass = NULL;
	LQLQDiscClass *parentKlass = NULL;
	LQLQDiscNetem *netem = NULL;

	g_assert(self != NULL);
	g_assert(LQL_IS_QDISC_NETEM(self));

	netem = LQL_QDISC_NETEM(self);

        if (netem->private->dispose_has_run) {
                return;
        }

	klass = LQL_QDISC_NETEM_GET_CLASS(self);
	parentKlass = g_type_class_peek_parent(klass);
	
	parentKlass->print(self);

	g_print("Type: Netem\n");
	g_print("Netem latency: %u ms\n", lql_qdisc_netem_get_latency(netem));
	g_print("Netem limit: %u packets\n", lql_qdisc_netem_get_limit(netem));
	g_print("Netem loss: %.2f%%\n", lql_qdisc_netem_get_loss(netem));
	g_print("Netem gap: %u\n", lql_qdisc_netem_get_gap(netem));
	g_print("Netem duplicate: %.2f%%\n", lql_qdisc_netem_get_duplicate(netem));
	g_print("Netem jitter: %u ms\n", lql_qdisc_netem_get_jitter(netem));
	g_print("Netem latency correlation: %.2f%%\n",
		lql_qdisc_netem_get_latencyCorrelation(netem));
	g_print("Netem loss correlation: %.2f%%\n",
		lql_qdisc_netem_get_lossCorrelation(netem));
	g_print("Netem duplicate correlation: %.2f%%\n",
		lql_qdisc_netem_get_duplicateCorrelation(netem));
}

void
lql_qdisc_netem_helper_printStats(LQLQDisc *self)
{
	LQLQDiscNetemClass *klass = NULL;
	LQLQDiscClass *parentKlass = NULL;
	LQLQDiscNetem *netem = NULL;

	g_assert(self != NULL);
	g_assert(LQL_IS_QDISC_NETEM(self));

	netem = LQL_QDISC_NETEM(self);

        if (netem->private->dispose_has_run) {
                return;
        }

	klass = LQL_QDISC_NETEM_GET_CLASS(self);
	parentKlass = g_type_class_peek_parent(klass);
	
	parentKlass->printStats(self);
}

/* static gboolean */
/* lql_qdisc_htb_helper_fill_xstats(LQLQDiscHTB *self, */
/* 				 const struct nlmsghdr *hdr) */
/* { */
/* 	const struct tc_htb_glob *glob = NULL; */

/* 	g_assert(self != NULL); */
/* 	g_assert(hdr != NULL); */

/* 	glob = nl_tcmsg_get_rta_val(hdr, TCA_HTB_INIT); */
/* 	if (glob == NULL) { */
/* 		return(FALSE); */
/* 	} */

/* 	/\* Filter for the QDisc we are looking for since there doesn't appear */
/* 	   to be a way to tell the kernel to only return a single QDisc. *\/ */
/*  	if (nl_tcmsg_get_handle(hdr) == */
/* 	    lql_element_get_handle(LQL_ELEMENT(self))) { */
/* 		self->private->directPackets = glob->direct_pkts; */
/* 	} */

/* 	return(TRUE); */
/* } */

gboolean
lql_qdisc_netem_helper_updateStats(LQLQDisc *self,
				   LQLCon *con)
{
	LQLQDiscNetemClass *klass = NULL;
	LQLQDiscClass *parentKlass = NULL;
	LQLQDiscNetem *netem = NULL;

	g_assert(self != NULL);
	g_assert(LQL_IS_QDISC_NETEM(self));

	netem = LQL_QDISC_NETEM(self);

        if (netem->private->dispose_has_run) {
                return(FALSE);
        }

	klass = LQL_QDISC_NETEM_GET_CLASS(self);
	parentKlass = g_type_class_peek_parent(klass);
	
	return(parentKlass->updateStats(self, con));
}

static guint32
lql_qdisc_netem_helper_per_to_u32(float percent)
{
	guint32 val = 0;

	/* Netem considers 100% to be 0xffffffff and 0% to be 0. */
	val = (percent / 100) * 0xffffffff;

	return(val);
}
