/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <unistd.h>
#include <string.h>

#include "debug_macros.h"
#include "nl.h"
#include "lql-con.h"
#include "lql-qdisc-sfq.h"
#include "lql-qdisc-sfq-private.h"

gboolean
lql_qdisc_sfq_helper_fill_from_nlhdr(LQLQDiscSFQ *qdisc,
				     const struct nlmsghdr *hdr)
{
	struct tc_sfq_qopt *opt = NULL;
	
	g_assert(qdisc != NULL);
	g_assert(hdr != NULL);

	lql_element_fill_from_nlhdr(LQL_ELEMENT(qdisc), hdr);
	
	opt = (struct tc_sfq_qopt *)nl_tcmsg_get_rta_val(hdr, TCA_OPTIONS);
	if (opt == NULL) {
		g_printerr("LQLQDiscSFQ:fill_from_nlhdr: Could not get options.\n");
		return(FALSE);
	}
	lql_qdisc_sfq_set_perturb(LQL_QDISC_SFQ(qdisc), opt->perturb_period);
	lql_qdisc_sfq_set_quantum(LQL_QDISC_SFQ(qdisc), opt->quantum);
	
	return(TRUE);
}

static gboolean
lql_qdisc_sfq_helper_do(LQLQDiscSFQ *qdisc,
			LQLCon *con,
			int flags)
{
	TcPacket *pkt = NULL;
	struct nlmsghdr *hdr = NULL;
	struct tc_sfq_qopt *opt = NULL;
	gchar *kind = "sfq";

	g_assert(con != NULL);
	g_assert(qdisc != NULL);

	pkt = nl_tcpacket_new();
	hdr = nl_tcpacket_get_nlmsg(pkt);
	
	nl_nlmsg_set_type(hdr, RTM_NEWQDISC);
	nl_nlmsg_set_flags(hdr, NLM_F_REQUEST | NLM_F_ACK | flags);
	nl_nlmsg_set_pid(hdr, getpid());
	nl_nlmsg_set_seq(hdr, lql_con_get_inc_sequence(LQL_CON(con)));

	nl_tcmsg_set_family(hdr, AF_UNSPEC);
	nl_tcmsg_set_ifindex(hdr, lql_element_get_ifIndex(LQL_ELEMENT(qdisc)));
	nl_tcmsg_set_parent(hdr, lql_element_get_parentHandle(LQL_ELEMENT(qdisc)));
	nl_tcmsg_set_handle(hdr, lql_element_get_handle(LQL_ELEMENT(qdisc)));
	
	opt = g_malloc0(sizeof(struct tc_sfq_qopt));
	opt->perturb_period = lql_qdisc_sfq_get_perturb(qdisc);
	opt->quantum = lql_qdisc_sfq_get_quantum(qdisc);

	if (!nl_tcpacket_add_rtattr(pkt, TCA_KIND, strlen(kind) + 1, kind)) {
		debug_print(5, "Could not add TCA_KIND");
	}
	if (!nl_tcpacket_add_rtattr(pkt, TCA_OPTIONS, sizeof(struct tc_sfq_qopt), opt)) {
		debug_print(5, "Could not add TCA_OPTIONS");
	}
	
	if (!nl_tcpacket_do_command(pkt, lql_con_get_nlcon(LQL_CON(con)))) {
		return(FALSE);
	}
	
	g_free(pkt);
	g_free(opt);
	
	return(TRUE);
}

gboolean
lql_qdisc_sfq_helper_add(LQLQDiscSFQ *qdisc,
			 LQLCon *con)
{
	return(lql_qdisc_sfq_helper_do(qdisc, con, NLM_F_CREATE | NLM_F_EXCL));
}

gboolean
lql_qdisc_sfq_helper_modify(LQLQDiscSFQ *qdisc,
			    LQLCon *con)
{
	return(FALSE);
}

void
lql_qdisc_sfq_helper_print(LQLQDisc *self)
{
	LQLQDiscSFQClass *klass = NULL;
	LQLQDiscClass *parentKlass = NULL;
	LQLQDiscSFQ *sfq = NULL;

	g_assert(self != NULL);
	g_assert(LQL_IS_QDISC_SFQ(self));

	sfq = LQL_QDISC_SFQ(self);

        if (sfq->private->dispose_has_run) {
                return;
        }

	klass = LQL_QDISC_SFQ_GET_CLASS(self);
	parentKlass = g_type_class_peek_parent(klass);
	
	parentKlass->print(self);

	g_print("Type: SFQ\n");
	g_print("SFQ perturb: %u\n", lql_qdisc_sfq_get_perturb(sfq));
	g_print("SFQ quantum: %u\n", lql_qdisc_sfq_get_quantum(sfq));
}
