#include <glib.h>

#include "lql-con.h"
#include "lql-interface.h"
#include "lql-qdisc-dsmark.h"
#include "lql-class-dsmark.h"

int
main(int argc,
     char *argv[])
{
	LQLCon *con = NULL;
	LQLQDiscDSMark *qdisc = NULL;
	LQLClassDSMark *class1 = NULL, *class2 = NULL;
	GList *list = NULL;
	LQLInterface *interface = NULL;

	g_type_init();

	con = lql_con_new();

	interface = lql_con_get_interface_by_name(con, "eth0");
	if (interface == NULL) {
		g_printerr("Could not get interface.\n");
		return(1);
	}

	/******************* Add ******************/

	g_print("Adding a root qdisc.\n");
	
	qdisc = lql_qdisc_dsmark_new();
	lql_qdisc_set_parent(LQL_QDISC(qdisc), NULL);
	lql_qdisc_set_id(LQL_QDISC(qdisc), 1);
	lql_qdisc_dsmark_set_indices(LQL_QDISC_DSMARK(qdisc), 64);
	lql_qdisc_dsmark_set_defaultIndex(LQL_QDISC_DSMARK(qdisc), 1);
	lql_qdisc_dsmark_set_setTCIndex(LQL_QDISC_DSMARK(qdisc), TRUE);
	lql_element_set_interface(LQL_ELEMENT(qdisc), interface);

	if (!lql_qdisc_add(LQL_QDISC(qdisc), con)) {
		g_printerr("Could not add qdisc.\n");
		return(1);
	}

	g_print("Adding class 1.\n");

	class1 = lql_class_dsmark_new();
	lql_class_set_parent(LQL_CLASS(class1), LQL_ELEMENT(qdisc));
	lql_class_set_id(LQL_CLASS(class1), 1);
	lql_element_set_interface(LQL_ELEMENT(class1), interface);
	lql_class_dsmark_set_mask(class1, 1);
	lql_class_dsmark_set_value(class1, 20);
	if (!lql_class_add(LQL_CLASS(class1), con)) {
		g_printerr("Line #%i could not add class.\n", __LINE__);
		return(1);
	}
	g_object_unref(class1);

	g_print("Adding class 2.\n");

	class1 = lql_class_dsmark_new();
	lql_class_set_parent(LQL_CLASS(class1), LQL_ELEMENT(qdisc));
	lql_class_set_id(LQL_CLASS(class1), 2);
	lql_element_set_interface(LQL_ELEMENT(class1), interface);
	lql_class_dsmark_set_mask(class1, 2);
	lql_class_dsmark_set_value(class1, 30);
	if (!lql_class_add(LQL_CLASS(class1), con)) {
		g_printerr("Line #%i could not add class.\n", __LINE__);
		return(1);
	}
	g_object_unref(class1);

	/***************** Verify QDisc ****************/

	g_print("Verifying successful qdisc add.\n");

	list = lql_con_get_qdiscs(con, interface);
	if (list == NULL) {
		g_printerr("Could not get qdisc list.\n");
		return(1);
	}

	if (g_list_length(list) != 1) {
		g_printerr("Wrong number of qdiscs returned.\n");
		return(1);
	}

	qdisc = LQL_QDISC_DSMARK(list->data);
	
	if (lql_element_get_handle(LQL_ELEMENT(qdisc)) != 65536) {
		g_printerr("Wrong ID number. Got: %u\n",
			   lql_element_get_handle(LQL_ELEMENT(qdisc)));
		return(1);
	}

	if (lql_element_get_ifIndex(LQL_ELEMENT(qdisc)) !=
	    lql_interface_get_ifIndex(interface)) {
		g_printerr("Wrong interface ID.\n");
		return(1);
	}

	if (lql_element_get_parentHandle(LQL_ELEMENT(qdisc)) != TC_H_ROOT) {
		g_printerr("Line #%i wrong parent handle.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_dsmark_get_indices(qdisc) != 64) {
		g_printerr("Wrong indices value.\n");
		return(1);
	}
	
	if (lql_qdisc_dsmark_get_defaultIndex(qdisc) != 1) {
		g_printerr("Wrong default index value.\n");
		return(1);
	}

	if (lql_qdisc_dsmark_get_setTCIndex(qdisc) != TRUE) {
		g_printerr("Line #%i Wrong setTCIndex value.\n", __LINE__);
		return(1);
	}

	g_list_free(list); list = NULL;

	/*************** Verify classes *****************/

	g_print("Verifying classes.\n");

	list = lql_con_get_classes(con, interface);
	if (list == NULL) {
		g_printerr("Line #%i could not get class list.\n", __LINE__);
		return(1);
	}
	
	if (g_list_length(list) != 2) {
		g_printerr("Line #%i wrong number of classes returned.\n", __LINE__);
		return(1);
	}

	/*********** class 1 ************/

	g_print("Class 1.\n");
	
	class1 = LQL_CLASS_DSMARK(list->data);
	class2 = LQL_CLASS_DSMARK(list->next->data);

/* 	if (lql_element_get_parentHandle(LQL_ELEMENT(class1)) != 4294967295) { */
/* 		g_printerr("Line #%i wrong parent. Got %u\n", */
/* 			   __LINE__, lql_element_get_parentHandle(LQL_ELEMENT(class1))); */
/* 		return(1); */
/* 	} */
	
	if (lql_class_get_id(LQL_CLASS(class1)) != 1) {
		g_printerr("Line #%i wrong id.\n", __LINE__);
		return(1);
	}
	
	if (lql_element_get_ifIndex(LQL_ELEMENT(class1)) !=
	    lql_interface_get_ifIndex(interface)) {
		g_printerr("Line #%i wrong interface index.\n", __LINE__);
		return(1);
	}
	
	if (lql_class_dsmark_get_mask(class1) != 1) {
		g_printerr("Line #%i wrong mask.\n", __LINE__);
		return(1);
	}
	
	if (lql_class_dsmark_get_value(class1) != 20) {
		g_printerr("Line #%i wrong value.\n", __LINE__);
		return(1);
	}

	/********** class 2 ***************/
	g_print("Class 2.\n");

/* 	if (lql_element_get_parentHandle(LQL_ELEMENT(class2)) != 4294967295) { */
/* 		g_printerr("Line #%i wrong parent.\n", __LINE__); */
/* 		return(1); */
/* 	} */
	
	if (lql_class_get_id(LQL_CLASS(class2)) != 2) {
		g_printerr("Line #%i wrong id.\n", __LINE__);
		return(1);
	}
	
	if (lql_element_get_ifIndex(LQL_ELEMENT(class2)) !=
	    lql_interface_get_ifIndex(interface)) {
		g_printerr("Line #%i wrong interface index.\n", __LINE__);
		return(1);
	}
	
	if (lql_class_dsmark_get_mask(class2) != 2) {
		g_printerr("Line #%i wrong mask.\n", __LINE__);
		return(1);
	}
	
	if (lql_class_dsmark_get_value(class2) != 30) {
		g_printerr("Line #%i wrong value.\n", __LINE__);
		return(1);
	}

	/********* Modify the classes ************/
	g_print("Modifying class parameters.\n");

	lql_class_set_priority(LQL_CLASS(class1), 1);
	lql_class_dsmark_set_mask(class1, 10);
	lql_class_dsmark_set_value(class1, 200);
	if (!lql_class_modify(LQL_CLASS(class1), con)) {
		g_printerr("Line #%i could not modify class.\n", __LINE__);
		return(1);
	}

	lql_class_set_priority(LQL_CLASS(class2), 2);
	lql_class_dsmark_set_mask(class2, 20);
	lql_class_dsmark_set_value(class2, 250);
	if (!lql_class_modify(LQL_CLASS(class2), con)) {
		g_printerr("Line #%i could not modify class.\n", __LINE__);
		return(1);
	}

	g_object_unref(class1);
	g_object_unref(class2);
	g_list_free(list); list = NULL;
	 
	/************* Verify changes *************/
	g_print("Verifying changes.\n");

	list = lql_con_get_classes(con, interface);
	if (list == NULL) {
		g_printerr("Line #%i could not get class list.\n", __LINE__);
		return(1);
	}
	
	if (g_list_length(list) != 2) {
		g_printerr("Line #%i wrong number of classes returned.\n", __LINE__);
		return(1);
	}
	
	class1 = LQL_CLASS_DSMARK(list->data);
	class2 = LQL_CLASS_DSMARK(list->next->data);
	
	/**** class1 ****/
	g_print("Class 1.\n");
	
	if (lql_element_get_ifIndex(LQL_ELEMENT(class1)) !=
	    lql_interface_get_ifIndex(interface)) {
		g_printerr("Line #%i wrong interface index.\n", __LINE__);
		return(1);
	}
	
	if (lql_class_dsmark_get_mask(class1) != 10) {
		g_printerr("Line #%i wrong mask.\n", __LINE__);
		return(1);
	}
	
	if (lql_class_dsmark_get_value(class1) != 200) {
		g_printerr("Line #%i wrong value.\n", __LINE__);
		return(1);
	}

	/**** class2 ****/
	
	if (lql_element_get_ifIndex(LQL_ELEMENT(class2)) !=
	    lql_interface_get_ifIndex(interface)) {
		g_printerr("Line #%i wrong interface index.\n", __LINE__);
		return(1);
	}
	
	if (lql_class_dsmark_get_mask(class2) != 20) {
		g_printerr("Line #%i wrong rate.\n", __LINE__);
		return(1);
	}
	
	if (lql_class_dsmark_get_value(class2) != 250) {
		g_printerr("Line #%i wrong ceil.\n", __LINE__);
		return(1);
	}

	/************** Deleting class 2 **************/
	g_print("Deleting class 2.\n");
	if (!lql_class_delete(LQL_CLASS(class2), con)) {
		g_printerr("Line #%i could not delete class.\n", __LINE__);
		return(1);
	}
	g_object_unref(class1);
	g_object_unref(class2);
	g_list_free(list);
	
	/************* Verify only one class left. **********/
	g_print("Verifying only one class left.\n");

	list = lql_con_get_classes(con, interface);
	if (list == NULL) {
		g_printerr("Line #%i could not get class list.\n", __LINE__);
		return(1);
	}
	
	if (g_list_length(list) != 1) {
		g_printerr("Line #%i wrong number of classes returned.\n", __LINE__);
		return(1);
	}
	
	class1 = LQL_CLASS_DSMARK(list->data);

        /*********** Delete last class **************/
	if (!lql_class_delete(LQL_CLASS(class1), con)) {
		g_printerr("Line #%i could not delete class.\n", __LINE__);
		return(1);
	}
	g_object_unref(class1);
	g_list_free(list);
	
	/********* Veryify no classes left ***************/
	g_print("Verifying that there are no classes left.\n");

	list = lql_con_get_classes(con, interface);
	if (list != NULL) {
		g_printerr("Line #%i could not get class list.\n", __LINE__);
		return(1);
	}

	/************ Delete the QDisc. ****************/
	g_print("Deleting qdisc.\n");
	
	if (!lql_qdisc_delete(LQL_QDISC(qdisc), con)) {
		g_printerr("Line #%i could not delete qdisc.\n", __LINE__);
		return(1);
	}

	return(0);
}
