#include <glib.h>

#include "lql-con.h"
#include "lql-interface.h"
#include "lql-qdisc-dsmark.h"

int
main(int argc,
     char *argv[])
{
	LQLCon *con = NULL;
	LQLQDiscDSMark *qdisc = NULL;
	GList *list = NULL;
	LQLInterface *interface = NULL;

	g_type_init();

	con = lql_con_new();

	interface = lql_con_get_interface_by_name(con, "eth0");
	if (interface == NULL) {
		g_printerr("Could not get interface.\n");
		return(1);
	}

	g_print("Adding a root qdisc.\n");

	qdisc = lql_qdisc_dsmark_new();
	lql_qdisc_set_parent(LQL_QDISC(qdisc), NULL);
	lql_qdisc_set_id(LQL_QDISC(qdisc), 1);
	lql_element_set_interface(LQL_ELEMENT(qdisc), interface);
	lql_qdisc_dsmark_set_indices(qdisc, 32);
	lql_qdisc_dsmark_set_setTCIndex(qdisc, FALSE);
	lql_qdisc_dsmark_set_defaultIndex(qdisc, 2);

	if (!lql_qdisc_add(LQL_QDISC(qdisc), con)) {
		g_printerr("Line #%i could not add qdisc.\n", __LINE__);
		return(1);
	}

	g_object_unref(qdisc);

	/************** Verify *******************/

	g_print("Verifying successful add.\n");

	list = lql_con_get_qdiscs(con, interface);
	if (list == NULL) {
		g_printerr("Could not get qdisc list.\n");
		return(1);
	}

	if (g_list_length(list) != 1) {
		g_printerr("Wrong number of qdiscs returned.\n");
		return(1);
	}

	qdisc = LQL_QDISC_DSMARK(list->data);

	if (lql_element_get_handle(LQL_ELEMENT(qdisc)) != 65536) {
		g_printerr("Wrong ID number. Got: %u\n",
			   lql_element_get_handle(LQL_ELEMENT(qdisc)));
		return(1);
	}

	if (lql_element_get_ifIndex(LQL_ELEMENT(qdisc)) !=
	    lql_interface_get_ifIndex(interface)) {
		g_printerr("Wrong interface ID.\n");
		return(1);
	}

	if (lql_element_get_parentHandle(LQL_ELEMENT(qdisc)) != TC_H_ROOT) {
		g_printerr("Line #%i wrong parent handle.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_dsmark_get_indices(qdisc) != 32) {
		g_printerr("Line #%i returned indices value is wrong. Got %u\n",
			   __LINE__, lql_qdisc_dsmark_get_indices(qdisc));
		return(1);
	}
	
	if (lql_qdisc_dsmark_get_setTCIndex(qdisc) != FALSE) {
		g_printerr("Line #%i returned setTCIndex value is wrong.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_dsmark_get_defaultIndex(qdisc) != 2) {
		g_printerr("Line #%i returned default value is wrong.\n", __LINE__);
		return(1);
	}

	g_print("Deleting QDisc.\n");
	
	if (!lql_qdisc_delete(LQL_QDISC(qdisc), con)) {
		g_printerr("Line #%i could not delete qdisc.\n", __LINE__);
		return(1);
	}

	return(0);
}
