#include <glib.h>

#include "lql-con.h"
#include "lql-qdisc.h"
#include "lql-qdisc-htb.h"

int
main(int argc,
     char *argv[])
{
	LQLCon *con = NULL;
	LQLQDiscHTB *htb = NULL;
	LQLInterface *interface = NULL;
	GList *list = NULL;
	
	g_type_init();
	
	con = lql_con_new();

	interface = lql_con_get_interface_by_name(con, "eth0");
	if (interface == NULL) {
		g_printerr("Could not get interface.\n");
		return(1);
	}

	g_print("Adding a root qdisc.\n");
	
	htb = lql_qdisc_htb_new();
	lql_qdisc_set_parent(LQL_QDISC(htb), NULL);
	lql_qdisc_set_id(LQL_QDISC(htb), 1);
	lql_qdisc_htb_set_defaultClass(LQL_QDISC_HTB(htb), 1);
	lql_qdisc_htb_set_rateToQuantum(LQL_QDISC_HTB(htb), 20);
	lql_element_set_interface(LQL_ELEMENT(htb), interface);

	if (!lql_qdisc_add(LQL_QDISC(htb), con)) {
		g_printerr("Could not add qdisc.\n");
		return(1);
	}

	g_object_unref(htb);

	g_print("Verifying successful add.\n");

	list = lql_con_get_qdiscs(con, interface);
	if (list == NULL) {
		g_printerr("Could not get qdisc list.\n");
		return(1);
	}

	if (g_list_length(list) != 1) {
		g_printerr("Wrong number of qdiscs returned.\n");
		return(1);
	}

	htb = LQL_QDISC_HTB(list->data);
	
	if (lql_element_get_handle(LQL_ELEMENT(htb)) != 65536) {
		g_printerr("Wrong ID number. Got: %u\n",
			   lql_element_get_handle(LQL_ELEMENT(htb)));
		return(1);
	}

	if (lql_element_get_ifIndex(LQL_ELEMENT(htb)) !=
	    lql_interface_get_ifIndex(interface)) {
		g_printerr("Wrong interface ID.\n");
		return(1);
	}

	if (lql_element_get_parentHandle(LQL_ELEMENT(htb)) != TC_H_ROOT) {
		g_printerr("Line #%i wrong parent handle.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_htb_get_defaultClass(htb) != 1) {
		g_printerr("Wrong default class.\n");
		return(1);
	}
	
	if (lql_qdisc_htb_get_rateToQuantum(htb) != 20) {
		g_printerr("Wrong rate to quantum.\n");
		return(1);
	}

	g_print("Deleting QDisc.\n");
	
	if (!lql_qdisc_delete(LQL_QDISC(htb), con)) {
		g_printerr("Line #%i could not delete qdisc.\n", __LINE__);
		return(1);
	}
	
	return(0);
}
