#include <glib.h>

#include "lql-con.h"
#include "lql-interface.h"
#include "lql-qdisc-netem.h"

int
main(int argc,
     char *argv[])
{
	LQLCon *con = NULL;
	LQLQDiscNetem *qdisc = NULL;
	GList *list = NULL;
	LQLInterface *interface = NULL;

	g_type_init();

	con = lql_con_new();

	interface = lql_con_get_interface_by_name(con, "eth0");
	if (interface == NULL) {
		g_printerr("Could not get interface.\n");
		return(1);
	}

	g_print("Adding a root qdisc.\n");

	qdisc = lql_qdisc_netem_new();
	lql_qdisc_set_parent(LQL_QDISC(qdisc), NULL);
	lql_qdisc_set_id(LQL_QDISC(qdisc), 1);
	lql_element_set_interface(LQL_ELEMENT(qdisc), interface);
	lql_qdisc_netem_set_latency(qdisc, 50);
	lql_qdisc_netem_set_limit(qdisc, 1000);
	lql_qdisc_netem_set_loss(qdisc, 3);
	lql_qdisc_netem_set_gap(qdisc, 5);
	lql_qdisc_netem_set_duplicate(qdisc, 5);
	lql_qdisc_netem_set_jitter(qdisc, 17);
	lql_qdisc_netem_set_lossCorrelation(qdisc, 7);
	lql_qdisc_netem_set_latencyCorrelation(qdisc, 11);
	lql_qdisc_netem_set_duplicateCorrelation(qdisc, 13);

	if (!lql_qdisc_add(LQL_QDISC(qdisc), con)) {
		g_printerr("Line #%i could not add qdisc.\n", __LINE__);
		return(1);
	}

	/****************** Verify *********************/
	g_print("Verifying successful add.\n");

	list = lql_con_get_qdiscs(con, interface);
	if (list == NULL) {
		g_printerr("Could not get qdisc list.\n");
		return(1);
	}

	if (g_list_length(list) != 1) {
		g_printerr("Wrong number of qdiscs returned.\n");
		return(1);
	}

	qdisc = LQL_QDISC_NETEM(list->data);

	if (lql_element_get_handle(LQL_ELEMENT(qdisc)) != 65536) {
		g_printerr("Wrong ID number. Got: %u\n",
			   lql_element_get_handle(LQL_ELEMENT(qdisc)));
		return(1);
	}

	if (lql_element_get_ifIndex(LQL_ELEMENT(qdisc)) !=
	    lql_interface_get_ifIndex(interface)) {
		g_printerr("Wrong interface ID.\n");
		return(1);
	}

	if (lql_element_get_parentHandle(LQL_ELEMENT(qdisc)) != TC_H_ROOT) {
		g_printerr("Line #%i wrong parent handle.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_netem_get_latency(qdisc) != 50) {
		g_printerr("Line #%i wrong latency value. Got %u\n",
			   __LINE__, lql_qdisc_netem_get_latency(qdisc));
		return(1);
	}

	if (lql_qdisc_netem_get_limit(qdisc) != 1000) {
		g_printerr("Line #%i wrong limit value. Got %u\n",
			   __LINE__, lql_qdisc_netem_get_limit(qdisc));
		return(1);
	}

	if (lql_qdisc_netem_get_loss(qdisc) != 3) {
		g_printerr("Line #%i wrong loss value. Got %f\n",
			   __LINE__, lql_qdisc_netem_get_loss(qdisc));
		return(1);
	}

	if (lql_qdisc_netem_get_gap(qdisc) != 5) {
		g_printerr("Line #%i wrong gap value.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_netem_get_duplicate(qdisc) != 5) {
		g_printerr("Line #%i wrong duplicate value. Got %f\n",
			   __LINE__, lql_qdisc_netem_get_duplicate(qdisc));
		return(1);
	}

	if (lql_qdisc_netem_get_jitter(qdisc) != 17) {
		g_printerr("Line #%i wrong jitter value.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_netem_get_lossCorrelation(qdisc) != 7) {
		g_printerr("Line #%i wrong loss correlation value. Got %f\n",
			   __LINE__, lql_qdisc_netem_get_lossCorrelation(qdisc));
		return(1);
	}

	if (lql_qdisc_netem_get_latencyCorrelation(qdisc) != 11) {
		g_printerr("Line #%i wrong latency correlation value. Got %f\n",
			   __LINE__, lql_qdisc_netem_get_latencyCorrelation(qdisc));
		return(1);
	}

	if (lql_qdisc_netem_get_duplicateCorrelation(qdisc) != 13) {
		g_printerr("Line #%i wrong duplicate correlation value. Got %f\n",
			   __LINE__, lql_qdisc_netem_get_duplicateCorrelation(qdisc));
		return(1);
	}
	
	/***************** Modify **********************/
	g_print("Modifying qdisc parameters.\n");

	lql_qdisc_netem_set_latency(qdisc, 89);
	lql_qdisc_netem_set_limit(qdisc, 1000);
	lql_qdisc_netem_set_loss(qdisc, 7);
	lql_qdisc_netem_set_gap(qdisc, 2);
	lql_qdisc_netem_set_duplicate(qdisc, 45);
	lql_qdisc_netem_set_jitter(qdisc, 67);
	lql_qdisc_netem_set_lossCorrelation(qdisc, 11);
	lql_qdisc_netem_set_latencyCorrelation(qdisc, 13);
	lql_qdisc_netem_set_duplicateCorrelation(qdisc, 17);

	if (!lql_qdisc_modify(LQL_QDISC(qdisc), con)) {
		g_print("line #%i could not modify qdisc.\n", __LINE__);
		return(1);
	}
	
	g_object_unref(qdisc);
	g_list_free(list);

	/*************** Verify ***********************/
	g_print("Verifying successful modification.\n");

	list = lql_con_get_qdiscs(con, interface);
	if (list == NULL) {
		g_printerr("Could not get qdisc list.\n");
		return(1);
	}

	if (g_list_length(list) != 1) {
		g_printerr("Wrong number of qdiscs returned.\n");
		return(1);
	}

	qdisc = LQL_QDISC_NETEM(list->data);

	if (lql_element_get_handle(LQL_ELEMENT(qdisc)) != 65536) {
		g_printerr("Wrong ID number. Got: %u\n",
			   lql_element_get_handle(LQL_ELEMENT(qdisc)));
		return(1);
	}

	if (lql_element_get_ifIndex(LQL_ELEMENT(qdisc)) !=
	    lql_interface_get_ifIndex(interface)) {
		g_printerr("Wrong interface ID.\n");
		return(1);
	}

	if (lql_element_get_parentHandle(LQL_ELEMENT(qdisc)) != TC_H_ROOT) {
		g_printerr("Line #%i wrong parent handle.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_netem_get_latency(qdisc) != 89) {
		g_printerr("Line #%i wrong latency value.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_netem_get_limit(qdisc) != 1000) {
		g_printerr("Line #%i wrong limit value.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_netem_get_loss(qdisc) != 7) {
		g_printerr("Line #%i wrong loss value.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_netem_get_gap(qdisc) != 2) {
		g_printerr("Line #%i wrong gap value.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_netem_get_duplicate(qdisc) != 45) {
		g_printerr("Line #%i wrong duplicate value.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_netem_get_jitter(qdisc) != 67) {
		g_printerr("Line #%i wrong jitter value.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_netem_get_lossCorrelation(qdisc) != 11) {
		g_printerr("Line #%i wrong loss correlation value. Got %f\n",
			   __LINE__, lql_qdisc_netem_get_lossCorrelation(qdisc));
		return(1);
	}

	if (lql_qdisc_netem_get_latencyCorrelation(qdisc) != 13) {
		g_printerr("Line #%i wrong latency correlation value. Got %f\n",
			   __LINE__, lql_qdisc_netem_get_latencyCorrelation(qdisc));
		return(1);
	}

	if (lql_qdisc_netem_get_duplicateCorrelation(qdisc) != 17) {
		g_printerr("Line #%i wrong duplicate correlation value. Got %f\n",
			   __LINE__, lql_qdisc_netem_get_duplicateCorrelation(qdisc));
		return(1);
	}

	/***************** Deleting ********************/

	g_print("Deleting QDisc.\n");
	
	if (!lql_qdisc_delete(LQL_QDISC(qdisc), con)) {
		g_printerr("Line #%i could not delete qdisc.\n", __LINE__);
		return(1);
	}

	g_object_unref(qdisc);

	g_object_unref(con);

	return(0);
}
