#include <glib.h>

#include "lql-con.h"
#include "lql-interface.h"
#include "lql-qdisc-pfifo.h"

int
main(int argc,
     char *argv[])
{
	LQLCon *con = NULL;
	LQLQDiscPFIFO *qdisc = NULL;
	GList *list = NULL;
	LQLInterface *interface = NULL;

	g_type_init();

	con = lql_con_new();

	interface = lql_con_get_interface_by_name(con, "eth0");
	if (interface == NULL) {
		g_printerr("Could not get interface.\n");
		return(1);
	}

	/*************** Add *****************/

	g_print("Adding a root qdisc.\n");
	
	qdisc = lql_qdisc_pfifo_new();
	lql_qdisc_set_parent(LQL_QDISC(qdisc), NULL);
	lql_qdisc_set_id(LQL_QDISC(qdisc), 1);
	lql_element_set_interface(LQL_ELEMENT(qdisc), interface);
	lql_qdisc_pfifo_set_queueLength(LQL_QDISC_PFIFO(qdisc), 8);

	if (!lql_qdisc_add(LQL_QDISC(qdisc), con)) {
		g_printerr("Line #%i could not add qdisc.\n", __LINE__);
		return(1);
	}

	g_object_unref(qdisc);

	/*************** Verify *****************/
	
	g_print("Verifying successful add.\n");

	list = lql_con_get_qdiscs(con, interface);
	if (list == NULL) {
		g_printerr("Could not get qdisc list.\n");
		return(1);
	}

	if (g_list_length(list) != 1) {
		g_printerr("Wrong number of qdiscs returned.\n");
		return(1);
	}

	qdisc = LQL_QDISC_PFIFO(list->data);
	
	if (lql_element_get_handle(LQL_ELEMENT(qdisc)) != 65536) {
		g_printerr("Wrong ID number. Got: %u\n",
			   lql_element_get_handle(LQL_ELEMENT(qdisc)));
		return(1);
	}

	if (lql_element_get_ifIndex(LQL_ELEMENT(qdisc)) !=
	    lql_interface_get_ifIndex(interface)) {
		g_printerr("Wrong interface ID.\n");
		return(1);
	}

	if (lql_element_get_parentHandle(LQL_ELEMENT(qdisc)) != TC_H_ROOT) {
		g_printerr("Line #%i wrong parent handle.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_pfifo_get_queueLength(qdisc) != 8) {
		g_printerr("Line #%i got wrong queue length.\n", __LINE__);
		return(1);
	}

	/*************** Modify *****************/

	g_print("Modifying qdisc parameters.\n");

	lql_qdisc_pfifo_set_queueLength(LQL_QDISC_PFIFO(qdisc), 12);

	g_print("Attempting to modify qdisc.\n");

	if (!lql_qdisc_modify(LQL_QDISC(qdisc), con)) {
		g_print("line #%i could not modify qdisc.\n", __LINE__);
		return(1);
	}
	
	g_object_unref(qdisc);
	g_list_free(list);

	/*************** Verify *****************/

	g_print("Verifying modification.\n");

	list = lql_con_get_qdiscs(con, interface);
	if (list == NULL) {
		g_printerr("Could not get qdisc list.\n");
		return(1);
	}

	if (g_list_length(list) != 1) {
		g_printerr("Wrong number of qdiscs returned.\n");
		return(1);
	}

	qdisc = LQL_QDISC_PFIFO(list->data);

	if (lql_element_get_handle(LQL_ELEMENT(qdisc)) != 65536) {
		g_printerr("Wrong ID number. Got: %u\n",
			   lql_element_get_handle(LQL_ELEMENT(qdisc)));
		return(1);
	}

	if (lql_element_get_ifIndex(LQL_ELEMENT(qdisc)) !=
	    lql_interface_get_ifIndex(interface)) {
		g_printerr("Wrong interface ID.\n");
		return(1);
	}

	if (lql_element_get_parentHandle(LQL_ELEMENT(qdisc)) != TC_H_ROOT) {
		g_printerr("Line #%i wrong parent handle.\n", __LINE__);
		return(1);
	}

	if (lql_qdisc_pfifo_get_queueLength(qdisc) != 12) {
		g_printerr("Line #%i got wrong queue length.\n", __LINE__);
		return(1);
	}

	g_print("Deleting QDisc.\n");
	
	if (!lql_qdisc_delete(LQL_QDISC(qdisc), con)) {
		g_printerr("Line #%i could not delete qdisc.\n", __LINE__);
		return(1);
	}

	g_object_unref(con);

	return(0);
}
